/* ============================================================================
System Name:	Totem Pole Bridgeless CCM PFC (TIDM-1007)

File Name:	  	pfc1ph3ilttpl.c

Author:			Manish Bhardwaj, 7/13/2018

Description:	This is the solution file, following is the description of
                other files that are used by this system
				<solution>.c -> solution source file
				<solution>.h -> solution header file
				<solution>_settings.h -> powerSUITE generated settings
				<boad name>_board.c -> board drivers source file
				<boad name>_board.h -> board drivers header file

                <optional>
 			    <solution>_cla.cla -> cla task file
                <solution>_cla.h -> cla task header file
===========================================================================  */
//#############################################################################
// $TI Release: TIDM_1007 v1.00.00.00 $
// $Release Date: Thu May 28 13:35:58 CDT 2020 $
// $Copyright:
// Copyright (C) 2020 Texas Instruments Incorporated - http://www.ti.com/
//
// ALL RIGHTS RESERVED
// $
//#############################################################################


//*****************************************************************************
// the includes
//*****************************************************************************

#include "pfc1ph3ilttpl.h"

//*********************** C2000 SYSTEM SW FRAMEWORK  ************************//


//*****************************************************************************
// globals
//*****************************************************************************

//---  State Machine Related ---

int16_t vTimer0[4];         // Virtual Timers slaved off CPU Timer 0 (A events)
int16_t vTimer1[4];         // Virtual Timers slaved off CPU Timer 1 (B events)
int16_t vTimer2[4];         // Virtual Timers slaved off CPU Timer 2 (C events)

// Variable declarations for state machine
void (*Alpha_State_Ptr)(void);  // Base States pointer
void (*A_Task_Ptr)(void);       // State pointer A branch
void (*B_Task_Ptr)(void);       // State pointer B branch
void (*C_Task_Ptr)(void);       // State pointer C branch

// State Machine function prototypes
//------------------------------------
// Alpha states
void A0(void);  //state A0
void B0(void);  //state B0

// A branch states
void A1(void);  //state A1

// B branch states
void B1(void);  //state B1
void B2(void);  //state B2
void B3(void);  //state B3

//--- System Related ---
#pragma SET_DATA_SECTION("controlVariables")
// RAMP to generate forced angle when grid is not present
RAMPGEN rgen1;
volatile float32_t sineAngle;

GI_STRUCT gi;

GV_STRUCT gv;
SPLL_1PH_SOGI spll1;

SPLL_1PH_NOTCH spll2;

SPLL_1PH_SOGI_FLL spll3;

NOTCH_2P2Z_STRUCT notch1, notch2;

NOTCH_2P2Z_STRUCT notch_50Hz, notch_60Hz;
NOTCH_2P2Z_STRUCT spll_notch_50Hz, spll_notch_60Hz;

// Sine analyzer block for RMS Volt, Curr and Power measurements
POWER_MEAS_SINE_ANALYZER sine_mains;

uint16_t guiPowerStageStart;
uint16_t guiPowerStageStop;
int16_t slewState;

volatile buildLevel build_Level;
volatile boardState board_State;
volatile boardStatus board_Status;

//TODO Control Variables
// Measurement Variables
// Inductor Current Measurement
float32_t iL1_sensed, iL2_sensed, iL3_sensed;
float32_t iL1_sensed_scaled, iL2_sensed_scaled, iL3_sensed_scaled;
 float32_t iL1_senseOffset, iL2_senseOffset, iL3_senseOffset;
// AC Current Measurement
float32_t ac_cur_sensed;
float32_t ac_cur_sensed_scaled;
float32_t ac_cur_senseOffset;
float32_t ac_cur_sensed_dc_average;

volatile float32_t thetaOffset;

// Output Voltage measurement
volatile float32_t vBus_sensed;
volatile float32_t vBus_sensedOffset;

//Input voltage measurement
volatile float32_t ac_L_sensed, ac_N_sensed, ac_vol_sensed;
volatile float32_t ac_L_senseOffset, ac_N_senseOffset, ac_vol_senseOffset;

//Filtered DC bus measurement
volatile float32_t vBusAvg;
volatile float32_t vRmsAvg;

volatile float32_t threshold_PZC1,threshold_PZC2,threshold_NZC1,threshold_NZC2;

// Reference variables
// current set point
volatile float32_t ac_cur_ref;
float32_t ac_cur_ref_inst;
float32_t ac_cur_ref_inst_prev;
volatile float32_t inductor_voltage_drop_feedforward;
// voltage set point
volatile float32_t vBusRef, vBusRefSlewed;
// 10 corresponds to 100ns dead time
uint32_t dbRED_SetValue;
uint32_t dbRED_SetValue_Temp;
uint32_t dbFED_SetValue;

uint32_t dbRED_SetValue1;
uint32_t dbRED_SetValue_Temp1;
uint32_t dbRED_SetValue2;
uint32_t dbRED_SetValue_Temp2;
uint32_t dbRED_SetValue3;
uint32_t dbRED_SetValue_Temp3;
uint32_t dbRED_Min;
uint32_t dbRED_Max;

volatile float32_t softstart_duty;
// Display Values
volatile float32_t guiVbus, guiVbusMax;
volatile float32_t guiVbus_threshold;
volatile float32_t guiACFreq;
volatile float32_t guiPrms;
volatile float32_t guiIrms;
volatile float32_t guiVrms, guiVrmsMax;
volatile float32_t guiVrms_threshold;
volatile float32_t guiVavg;
volatile float32_t guiVin;
volatile float32_t guiIi;
volatile float32_t guiIo;
volatile float32_t guiPowerFactor;
volatile float32_t guiVA;
volatile float32_t guiFreqAvg;
volatile float32_t guiVema;

volatile float32_t guiVrmsEMAvg;
volatile int32_t autoStartSlew;
// Phase-Shedding Variables
volatile float32_t iref1;
volatile float32_t iref2;
volatile float32_t deltaIref;

// Duty variables
// per unit duty value
volatile float32_t dutyPU,dutyPU_DC;
// duty value written into the PWM register
volatile int32_t dutyPWMREG,dutyPWMREGcompl;
// PWM TBPRD Half
volatile float32_t halfPWMPeriod;

// Flags for clearing trips and closing the loops and the Relay
volatile int32_t closeGiLoop,closeGvLoop, clearTrip, rlyConnect,
                 firstTimeGvLoop;

// Flags for detecting ZCD
volatile float32_t acSine, acSinePrev;
volatile float32_t acCosine, acCosinePrev;
volatile int32_t zeroCrossDetectFlag;

volatile int32_t tempChange=0;
volatile int32_t period;

// Stand Alone Flash Image Instrumentation
// GPIO toggles for different states
int16_t ledBlinkCnt1, ledBlinkCnt2;
int16_t timer1;

volatile pwmSwState pwm_SwState;
volatile interleavingState interleaving_State;
volatile int32_t state_slew=0;
volatile int32_t state_slew_max=10;
//volatile int32_t state_slew_max=20;
volatile float32_t gi_out, gv_out;

volatile float32_t vBusSensedBuff[10];
volatile float32_t vBus_sensedFiltered;
volatile int32_t vBusSensedBuffIndex=0;

volatile float32_t vBus_sensedFiltered_notch1,vBus_sensedFiltered_notch2;

volatile float32_t gi_out_prev=0;

volatile float32_t ac_vol_sensed_buff[5];
volatile int32_t ac_vol_sensed_buff_index=0;
volatile int32_t ac_sign_filtered;

volatile float32_t spll_sine,spll_cosine;
volatile float32_t voltage_error;

// 1 if using non linear voltage loop, 0 otherwise
volatile uint32_t nonLinearVoltageLoopFlag;
volatile uint32_t nonLinearVoltageLoopFlagDelay;

volatile uint32_t softStartDeadBandFED;

// Variables used to calibrate measurement offsets
//Offset filter coefficient K1: 0.05/(T+0.05);
float32_t k1 = 0.998;
//Offset filter coefficient K2: T/(T+0.05)
float32_t k2 = 0.001999;

float32_t offset165;

volatile float32_t phase1ScaleFactor=1.0;
volatile float32_t phase2ScaleFactor=1.0;
volatile float32_t phase3ScaleFactor=1.0;

#pragma SET_DATA_SECTION()

// datalogger
DLOG_4CH dLog1;
float32_t dBuff1[100], dBuff2[100], dBuff3[100], dBuff4[100];
float32_t dVal1, dVal2, dVal3, dVal4;

//--- SFRA Related Variables ---

#if SFRA_TYPE!=SFRA_DISABLED
float32_t plantMagVect[SFRA_FREQ_LENGTH];
float32_t plantPhaseVect[SFRA_FREQ_LENGTH];
float32_t olMagVect[SFRA_FREQ_LENGTH];
float32_t olPhaseVect[SFRA_FREQ_LENGTH];
float32_t freqVect[SFRA_FREQ_LENGTH];
#endif

SFRA_F32 sfra1;

// Counter
int16_t i;

//TODO Main
void main(void)
{
	// This routine sets up the basic device configuration such as
    // initializing PLL, copying code from FLASH to RAM,
	// this routine will also initialize the CPU timers that are used in
    // the background task for this system
	setupDevice();

	// Tasks State-machine init
	Alpha_State_Ptr = &A0;
	A_Task_Ptr = &A1;
	B_Task_Ptr = &B1;

	// Stop all PWM mode clock
	disablePWMCLKCounting();

	setupPFC1ph3ilttplPWM(LOW_FREQ_PWM_BASE,HIGH_FREQ_PWM1_BASE,
	                      HIGH_FREQ_PWM2_BASE,HIGH_FREQ_PWM3_BASE,
	                      PFC_PWM_PERIOD,
	                      HIGH_FREQ_PWM_DEADBAND_RED_COUNT,
	                      HIGH_FREQ_PWM_DEADBAND_FED_COUNT);

	// power up ADC on the device
	setupADC();


    //Profiling GPIO
	setupProfilingGPIO();

	//configure LED GPIO
	setupLEDGPIO();

	//configure Relay GPIO
	setupRelayGPIO();

	//configure gain change GPIO
	setupGainChangeGPIO();
	gainLowSetforHighCurrent();

	// Initialize global variables generic to the board like ones used to
	// read current values etc
	globalVariablesInit();

#if SFRA_TYPE==SFRA_DISABLED
#else
	setupSFRA();
#endif

	// Enable PWM Clocks
	enablePWMCLKCounting();

	setupEPWMtoTriggerADCSOC(LOW_FREQ_PWM_BASE);
	setupEPWMtoTriggerADCSOC(HIGH_FREQ_PWM1_BASE);
	setupEPWMtoTriggerADCSOC(HIGH_FREQ_PWM2_BASE);
	setupEPWMtoTriggerADCSOC(HIGH_FREQ_PWM3_BASE);

	// Offset Calibration Routine
	calibrateOffset(&ac_cur_senseOffset, &iL1_senseOffset, &iL2_senseOffset,
	                &iL3_senseOffset , k1 , k2);
	// note the value below is determined by looking at the DC component
	ac_cur_senseOffset=0.5025f;
	iL1_senseOffset=0.5f;
	iL2_senseOffset=0.5f;
	iL3_senseOffset=0.5f;

	// clear any spurious flags in the SDFM module
	// setup protection and trips for the board
	setupBoardProtection(LOW_FREQ_PWM_BASE,HIGH_FREQ_PWM1_BASE,
	                     HIGH_FREQ_PWM2_BASE,HIGH_FREQ_PWM3_BASE,I_TRIP_LIMIT,
	                     I_MAX_SENSE);

	// safe to setup PWM pins
	setPinsAsPWM();

	// ISR Mapping
	setupInterrupt();

// IDLE loop. Just sit and loop forever, periodically will branch into A0-A3,
// B0-B3, C0-C3 tasks
// Frequency of this branching is set in setupDevice routine:
	for(;;)  //infinite loop
	{
		// State machine entry & exit point
		//===========================================================
		(*Alpha_State_Ptr)();	// jump to an Alpha state (A0,B0,...)
		//===========================================================
	}
} //END MAIN CODE



//TODO control ISR Code
#if CONTROL_RUNNING_ON == C28x_CORE
interrupt void controlISR(void)
{
    pfcControlCode();
	clearInterrupt(C28x_CONTROLISR_INTERRUPT_PIE_GROUP_NO);
}// MainISR Ends Here
#endif

//TODO 10Khz ISR Code
#if INSTRUMENTATION_ISR_RUNNING_ON == C28x_CORE
interrupt void tenKHzISR(void)
{
    voltageLoopPlusInstrumentationCode();
}
#endif

//=============================================================================
//	STATE-MACHINE SEQUENCING AND SYNCRONIZATION FOR SLOW BACKGROUND TASKS
//=============================================================================

//--------------------------------- FRAME WORK --------------------------------
void A0(void)
{
	// loop rate synchronizer for A-tasks
	if(GET_TASK_A_TIMER_OVERFLOW_STATUS == 1)
	{
		CLEAR_TASK_A_TIMER_OVERFLOW_FLAG;	// clear flag

		//-----------------------------------------------------------
		(*A_Task_Ptr)();		// jump to an A Task (A1,A2,A3,...)
		//-----------------------------------------------------------

		vTimer0[0]++;			// virtual timer 0, instance 0 (spare)

	}



	Alpha_State_Ptr = &B0;		// Comment out to allow only A tasks
}

void B0(void)
{
	// loop rate synchronizer for B-tasks
	if(GET_TASK_B_TIMER_OVERFLOW_STATUS  == 1)
	{
		CLEAR_TASK_B_TIMER_OVERFLOW_FLAG;				// clear flag

		//-----------------------------------------------------------
		(*B_Task_Ptr)();		// jump to a B Task (B1,B2,B3,...)
		//-----------------------------------------------------------
		vTimer1[0]++;			// virtual timer 1, instance 0 (spare)
	}

	Alpha_State_Ptr = &A0;		// Allow C state tasks
}

//=============================================================================
//	A - TASKS (executed in every 1 msec)
//=============================================================================
//--------------------------------------------------------
void A1(void)
//--------------------------------------------------------
{
#if SFRA_TYPE!=SFRA_DISABLED
    SFRA_F32_runBackgroundTask(&sfra1);
    SFRA_GUI_runSerialHostComms(&sfra1);
#endif

	//-------------------
	//the next time CpuTimer0 'counter' reaches Period value go to A2
	A_Task_Ptr = &A1;
	//-------------------
}

//=============================================================================
//	B - TASKS (executed in every 5 msec)
//=============================================================================

//----------------------------------- USER ------------------------------------

//----------------------------------------
void B1(void)
//----------------------------------------
{
	updateBoardStatus();
#if INCR_BUILD==1
	#if DC_CHECK==1
        #if CONTROL_RUNNING_ON == CLA_CORE
            build_Level.enum_buildLevel=BuildLevel1_OpenLoop_DC_CLA;
        #else
            build_Level.enum_buildLevel=BuildLevel1_OpenLoop_DC;
        #endif
	#else
        #if CONTROL_RUNNING_ON == CLA_CORE
            build_Level.enum_buildLevel=BuildLevel1_OpenLoop_AC_CLA;
        #else
            build_Level.enum_buildLevel=BuildLevel1_OpenLoop_AC;
        #endif
	#endif
#elif INCR_BUILD==2
	#if DC_CHECK==1
        #if CONTROL_RUNNING_ON == CLA_CORE
            build_Level.enum_buildLevel=BuildLevel2_CurrentLoop_DC_CLA;
        #else
            build_Level.enum_buildLevel=BuildLevel2_CurrentLoop_DC;
        #endif
	#else
        #if CONTROL_RUNNING_ON == CLA_CORE
            build_Level.enum_buildLevel=BuildLevel2_CurrentLoop_AC_CLA;
        #else
            build_Level.enum_buildLevel=BuildLevel2_CurrentLoop_AC;
        #endif
	#endif
#elif INCR_BUILD==3
	#if DC_CHECK==1
       #if CONTROL_RUNNING_ON == CLA_CORE
	        build_Level.enum_buildLevel=BuildLevel3_VoltageLoop_DC_CLA;
       #else
	        build_Level.enum_buildLevel=BuildLevel3_VoltageLoop_DC;
    #endif
	#else
        #if CONTROL_RUNNING_ON == CLA_CORE
	        build_Level.enum_buildLevel=BuildLevel3_VoltageLoop_AC_CLA;
        #else
	        build_Level.enum_buildLevel=BuildLevel3_VoltageLoop_AC;
        #endif
	#endif
#elif INCR_BUILD==4
        #if DC_CHECK==1
            #if CONTROL_RUNNING_ON == CLA_CORE
                build_Level.enum_buildLevel=BuildLevel4_OpenLoop_DC_CLA;
            #else
                build_Level.enum_buildLevel=BuildLevel4_OpenLoop_DC;
            #endif
        #else
            #if CONTROL_RUNNING_ON == CLA_CORE
                build_Level.enum_buildLevel=BuildLevel4_OpenLoop_AC_CLA;
            #else
                build_Level.enum_buildLevel=BuildLevel4_OpenLoop_AC;
            #endif
        #endif
#elif INCR_BUILD==5
        #if DC_CHECK==1
            #if CONTROL_RUNNING_ON == CLA_CORE
                build_Level.enum_buildLevel=BuildLevel5_CurrentLoop_DC_CLA;
            #else
                build_Level.enum_buildLevel=BuildLevel5_CurrentLoop_DC;
            #endif
        #else
            #if CONTROL_RUNNING_ON == CLA_CORE
                build_Level.enum_buildLevel=BuildLevel5_CurrentLoop_AC_ResLoad_CLA;
            #else
                build_Level.enum_buildLevel=BuildLevel5_CurrentLoop_AC_ResLoad;
            #endif
        #endif
#elif INCR_BUILD==6
        #if DC_CHECK==1
            #if CONTROL_RUNNING_ON == CLA_CORE
                build_Level.enum_buildLevel=BuildLevel6_CurrentLoop_DC_CLA;
            #else
                build_Level.enum_buildLevel=BuildLevel6_CurrentLoop_DC;
            #endif
        #else
            #if CONTROL_RUNNING_ON == CLA_CORE
                build_Level.enum_buildLevel=BuildLevel6_CurrentLoop_AC_GridConnected_CLA;
            #else
                build_Level.enum_buildLevel=BuildLevel6_CurrentLoop_GridConnected;
            #endif
        #endif
#endif

    computeNotchFltrCoeff( &spll_notch_60Hz, (float32_t)(CONTROL_ISR_FREQUENCY) ,
                           (float32_t)(60*2.0f) , 0.25f, 0.00001f);

	//-----------------
	//the next time CpuTimer1 'counter' reaches Period value go to B2
	B_Task_Ptr = &B2;
	//-----------------
}

//----------------------------------------
void B2(void) //  SPARE
//----------------------------------------
{

// Threshold setting for guiVbus and guiVrm to initiate autoStart (Inverter mode).
#if (INCR_BUILD==4 || INCR_BUILD==5)
    {
        guiVbus_threshold = 110;
        guiVrms_threshold = 0;
    }
#elif (INCR_BUILD==6)
    {
        guiVbus_threshold = 340;
        guiVrms_threshold = 75;
    }
#endif

#if DC_CHECK==1
	if(autoStartSlew<100)
	{
	    autoStartSlew++;
	}
#else
	// Auto Start for Inverter mode
    #if (INCR_BUILD > 3)
        if(guiVbus > guiVbus_threshold && guiVrms >= guiVrms_threshold )
        {
            if(autoStartSlew<5)
            {
                autoStartSlew++;
            }

            if(autoStartSlew==2)
            {


                DINT;

                if(sine_mains.acFreq>55)
                {
                    sine_mains.acFreq=60;
                }
                else
                {
                    sine_mains.acFreq=50;
                }

                SPLL_1PH_SOGI_config(&spll1,
                                     sine_mains.acFreq,
                                     CONTROL_ISR_FREQUENCY,
                                     (float32_t)(222.2862),
                                     (float32_t)(-222.034));

                SPLL_1PH_NOTCH_config(&spll2,
                                      sine_mains.acFreq,
                                      CONTROL_ISR_FREQUENCY,
                                      (float32_t)(222.2862),
                                      (float32_t)(-222.034),
                                      (float32_t) 0.25,
                                      (float32_t) 0.00001);

                SPLL_1PH_SOGI_FLL_config(&spll3,
                                         sine_mains.acFreq,
                                         CONTROL_ISR_FREQUENCY,
                                         (float32_t)(222.2862),
                                         (float32_t)(-222.034),
                                         (float32_t) 0.5,
                                         (float32_t) 20000);

                computeNotchFltrCoeff( &notch1,
                                       (float32_t)(INSTRUMENTATION_ISR_FREQUENCY),
                                       (float32_t)(sine_mains.acFreq*2.0) ,
                                       0.25f,
                                       0.00001f);

                computeNotchFltrCoeff( &notch2,
                                       (float32_t)(INSTRUMENTATION_ISR_FREQUENCY) ,
                                       (float32_t)(sine_mains.acFreq*2.0) ,
                                       0.25f,
                                       0.00001f);

                dutyPU=0.01f;

                AQ_SW_FORCE_PWMxA_LOW_PWMxB_LOW(LOW_FREQ_PWM_BASE);

                SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);

                SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);

                SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
                EINT;

                nonLinearVoltageLoopFlag=0;

            }

            if(autoStartSlew==3)
            {
                if(guiVrms<150 )
                {
                    thetaOffset=LOW_LINE_INPUT_CAP_COMP_ADJUST;
                }
                else
                {
                    thetaOffset=HIGH_LINE_INPUT_CAP_COMP_ADJUST;
                }

                DINT;
                clearPWMTripFlags(LOW_FREQ_PWM_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM1_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM2_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM3_BASE);
                board_Status.enum_boardStatus=boardStatus_NoFault;
                EINT;

            }
            if(autoStartSlew==1)
            {
                board_Status.enum_boardStatus=boardStatus_Idle;
                pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
            }



        }
        else
        {
            rlyConnect=0;
            autoStartSlew=0;
            pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
            closeGiLoop=0;
            closeGvLoop=0;

        }

    // Auto Start for PFC mode
    #else
        if(guiVrms > 75.0f )
        {
            if(autoStartSlew<5)
            {
                autoStartSlew++;
            }

            if(autoStartSlew==2)
            {


                DINT;

                if(sine_mains.acFreq>55)
                {
                    sine_mains.acFreq=60;
                }
                else
                {
                    sine_mains.acFreq=50;
                }

                SPLL_1PH_SOGI_config(&spll1,
                                     sine_mains.acFreq,
                                     CONTROL_ISR_FREQUENCY,
                                     (float32_t)(222.2862),
                                     (float32_t)(-222.034));

                SPLL_1PH_NOTCH_config(&spll2,
                                      sine_mains.acFreq,
                                      CONTROL_ISR_FREQUENCY,
                                      (float32_t)(222.2862),
                                      (float32_t)(-222.034),
                                      (float32_t) 0.25,
                                      (float32_t) 0.00001);

                SPLL_1PH_SOGI_FLL_config(&spll3,
                                         sine_mains.acFreq,
                                         CONTROL_ISR_FREQUENCY,
                                         (float32_t)(222.2862),
                                         (float32_t)(-222.034),
                                         (float32_t) 0.5,
                                         (float32_t) 20000);

                computeNotchFltrCoeff( &notch1,
                                       (float32_t)(INSTRUMENTATION_ISR_FREQUENCY),
                                       (float32_t)(sine_mains.acFreq*2.0) ,
                                       0.25f,
                                       0.00001f);

                computeNotchFltrCoeff( &notch2,
                                       (float32_t)(INSTRUMENTATION_ISR_FREQUENCY) ,
                                       (float32_t)(sine_mains.acFreq*2.0) ,
                                       0.25f,
                                       0.00001f);

                dutyPU=0.01f;

                AQ_SW_FORCE_PWMxA_LOW_PWMxB_LOW(LOW_FREQ_PWM_BASE);

                SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);

                SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);

                SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
                EINT;

                nonLinearVoltageLoopFlag=0;

            }

            if(autoStartSlew==3)
            {
                if(guiVrms<150 )
                {
                    thetaOffset=LOW_LINE_INPUT_CAP_COMP_ADJUST;
                }
                else
                {
                    thetaOffset=HIGH_LINE_INPUT_CAP_COMP_ADJUST;
                }

                DINT;
                clearPWMTripFlags(LOW_FREQ_PWM_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM1_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM2_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM3_BASE);
                board_Status.enum_boardStatus=boardStatus_NoFault;
                EINT;

            }
            if(autoStartSlew==1)
            {
                board_Status.enum_boardStatus=boardStatus_Idle;
                pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
            }



        }
        else
        {
            rlyConnect=0;
            autoStartSlew=0;
            pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
            closeGiLoop=0;
            closeGvLoop=0;

        }
    #endif
#endif

#if DC_CHECK==1
	if(autoStartSlew==100)
#else
	if(autoStartSlew==4)
#endif
	{

		DINT;

		vBusRefSlewed=0;

		gv.i6=0;
		gv.i10=0;

		gi.i10=0;
		gi.i6=0;

		closeGiLoop=0;
		closeGvLoop=0;

		clearTrip=1;
		firstTimeGvLoop=1;

		state_slew=0;
		softstart_duty=0;

		vBusRef=VBUS_TYPICAL/VDCBUS_MAX_SENSE;

		ac_cur_ref=0.03f;

		ac_cur_ref_inst=0;
		ac_cur_ref_inst_prev=0;

		acSine=0;
		acSinePrev=0;
		zeroCrossDetectFlag=0;

        #if INCR_BUILD >= 4
		// In inverter mode, it will not automatically start dc/ac conversion even after the auto start process
		// pwmSwState has to be manually changed from defaultState to normalOperation by user
            pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
        // The negative sign in ac_cur_ref represents the reverse power flow in inverter mode
            ac_cur_ref=-0.03f;
        #else
        // In PFC mode, it will automatically start ac/dc conversion after the auto start process
            pwm_SwState.enum_pwmSwState=pwmSwState_normalOperation;
        #endif

		EINT;

		autoStartSlew++;

#if DC_CHECK==1

		clearPWMTripFlags(LOW_FREQ_PWM_BASE);
		clearPWMTripFlags(HIGH_FREQ_PWM1_BASE);
		clearPWMTripFlags(HIGH_FREQ_PWM2_BASE);
		clearPWMTripFlags(HIGH_FREQ_PWM3_BASE);

		rlyConnect=1;
		closeGiLoop=1;

#endif

	}

	//-----------------
	//the next time CpuTimer1 'counter' reaches Period value go to B3
	B_Task_Ptr = &B3;
	//-----------------
}

//----------------------------------------
void B3(void) //  SPARE
//----------------------------------------
{
	toggleLED();
	//-----------------
	//the next time CpuTimer1 'counter' reaches Period value go to B1
	B_Task_Ptr = &B1;
	//-----------------
}

//TODO setupSFRA
#if SFRA_TYPE!=SFRA_DISABLED
void setupSFRA(void)
{
    SFRA_F32_reset(&sfra1);

    SFRA_F32_config(&sfra1,
                    SFRA_ISR_FREQ,
                    SFRA_AMPLITUDE,
                    SFRA_FREQ_LENGTH,
                    SFRA_FREQ_START,
                    SFRA_FREQ_STEP_MULTIPLY,
                    plantMagVect,
                    plantPhaseVect,
                    olMagVect,
                    olPhaseVect,
                    NULL,
                    NULL,
                    freqVect,
                    1);

    SFRA_F32_resetFreqRespArray(&sfra1);

	// Re-initialize the frequency array to make SFRA sweep go fast
	i=0;

	#if SFRA_TYPE==SFRA_CURRENT// current loop
		sfra1.freqVect[i++]=SFRA_FREQ_START;
		for(;i<sfra1.vecLength;i++)
		{
			if(sfra1.freqVect[i-1]<10)
				sfra1.freqVect[i]=sfra1.freqVect[i-1]+2;
			else if(sfra1.freqVect[i-1]<50)
				sfra1.freqVect[i]=sfra1.freqVect[i-1]+10;
			else
				sfra1.freqVect[i]=sfra1.freqVect[i-1]*sfra1.freqStep;
		}
	#else
		sfra1.freqVect[0]=2;
		sfra1.freqVect[1]=4;
		sfra1.freqVect[2]=6;
		sfra1.freqVect[3]=8;
		sfra1.freqVect[4]=10;
		sfra1.freqVect[5]=12;
		sfra1.freqVect[6]=14;
		sfra1.freqVect[7]=16;
		sfra1.freqVect[8]=18;
		sfra1.freqVect[9]=20;
		sfra1.freqVect[10]=22;
		sfra1.freqVect[11]=24;
		sfra1.freqVect[12]=26;
		sfra1.freqVect[13]=28;
		sfra1.freqVect[14]=30;
		sfra1.freqVect[15]=35;
		sfra1.freqVect[16]=40;
		sfra1.freqVect[17]=45;
		sfra1.freqVect[18]=55;
		sfra1.freqVect[19]=65;
		sfra1.freqVect[20]=70;
		sfra1.freqVect[21]=80;
		sfra1.freqVect[22]=90;
		sfra1.freqVect[23]=130;
		sfra1.freqVect[24]=140;
		sfra1.freqVect[25]=150;
		sfra1.freqVect[26]=160;
		sfra1.freqVect[27]=170;
		sfra1.freqVect[28]=210;
		sfra1.freqVect[29]=250;
	#endif

    //
    //configures the SCI channel for communication with SFRA host GUI
    //to change SCI channel change #defines in the settings.h file
    //the GUI also changes a LED status, this can also be changed with #define
    //in the file pointed to above
    //
    SFRA_GUI_config(SFRA_GUI_SCI_BASE,
                    SCI_VBUS_CLK,
                    SFRA_GUI_SCI_BAUDRATE,
                    SFRA_GUI_SCIRX_GPIO,
                    SFRA_GUI_SCIRX_GPIO_PIN_CONFIG,
                    SFRA_GUI_SCITX_GPIO,
                    SFRA_GUI_SCITX_GPIO_PIN_CONFIG,
                    SFRA_GUI_LED_INDICATOR,
                    SFRA_GUI_LED_GPIO,
                    SFRA_GUI_LED_GPIO_PIN_CONFIG,
                    &sfra1,
                    1);
}
#endif


void globalVariablesInit(void)
{

	RAMPGEN_reset(&rgen1);
	RAMPGEN_config(&rgen1,
	               CONTROL_ISR_FREQUENCY,
	               AC_FREQ);

	gi.Kp=GI_PI_KP;
	gi.Ki=GI_PI_KI;
	gi.Umax=GI_PI_MAX;
	gi.Umin=GI_PI_MIN;
	gi.i10=0;
	gi.i6=0;

	gv.Ki=GV_PI_KI;
	gv.Kp=GV_PI_KP;
	gv.Umax=GV_PI_MAX;
	gv.Umin=GV_PI_MIN;
	gv.i10=0;
	gv.i6=0;

	SPLL_1PH_SOGI_reset(&spll1);
	SPLL_1PH_SOGI_config(&spll1,
	                     AC_FREQ,
	                     CONTROL_ISR_FREQUENCY,
	                     (float32_t)(222.2862),
	                     (float32_t)(-222.034));

	SPLL_1PH_NOTCH_reset(&spll2);
    SPLL_1PH_NOTCH_config(&spll2,
                          AC_FREQ,
                          CONTROL_ISR_FREQUENCY,
                          (float32_t)(222.2862),
                          (float32_t)(-222.034),
                          (float32_t) 0.25,
                          (float32_t) 0.00001);

    SPLL_1PH_SOGI_FLL_reset(&spll3);
    SPLL_1PH_SOGI_FLL_config(&spll3,
                             AC_FREQ,
                             CONTROL_ISR_FREQUENCY,
                             (float32_t)(222.2862),
                             (float32_t)(-222.034),
                             (float32_t) 0.5,
                             (float32_t) 20000);

	//sine analyzer initialization
	POWER_MEAS_SINE_ANALYZER_reset(&sine_mains);
	POWER_MEAS_SINE_ANALYZER_config(&sine_mains,
	                                INSTRUMENTATION_ISR_FREQUENCY,
	                                (float32_t)0.08,
	                                (float32_t)GRID_MAX_FREQ,
	                                (float32_t)GRID_MIN_FREQ);

	computeNotchFltrCoeff( &notch1, (float32_t)(INSTRUMENTATION_ISR_FREQUENCY),
	                       (float32_t)(AC_FREQ*2.0) , 0.25f, 0.00001f);
	computeNotchFltrCoeff( &notch2, (float32_t)(INSTRUMENTATION_ISR_FREQUENCY),
	                       (float32_t)(AC_FREQ*2.0) , 0.25f, 0.00001f);
	notch1.x1=0;
	notch1.x2=0;
	notch2.x1=0;
	notch2.x2=0;

	computeNotchFltrCoeff( &notch_50Hz, (float32_t)(INSTRUMENTATION_ISR_FREQUENCY),
	                       (float32_t)(50*2.0) , 0.25f, 0.00001f);
	computeNotchFltrCoeff( &notch_60Hz, (float32_t)(INSTRUMENTATION_ISR_FREQUENCY),
	                       (float32_t)(60*2.0) , 0.25f, 0.00001f);

	computeNotchFltrCoeff( &spll_notch_50Hz, (float32_t)(CONTROL_ISR_FREQUENCY) ,
	                       (float32_t)(50*2.0) , 0.25f, 0.00001f);
	computeNotchFltrCoeff( &spll_notch_60Hz, (float32_t)(CONTROL_ISR_FREQUENCY) ,
	                       (float32_t)(60*2.0) , 0.25f, 0.00001f);

	DLOG_4CH_reset(&dLog1);
	DLOG_4CH_config(&dLog1,
                    &dVal1,&dVal2,&dVal3,&dVal4,
                    dBuff1,dBuff2,dBuff3,dBuff4,
                    100, 0.1f, 5);

	guiPowerStageStart=0;
	guiPowerStageStop=0;

	// Gui Variables
	guiVbus=0;
	guiACFreq=0;
	guiPrms=0;
	guiIrms=0;
	guiVrms=0;
	guiVrmsMax=270;
	guiVavg=0;
	guiFreqAvg=0;
	guiVema=0;
	guiVbusMax=440;
	guiVrmsEMAvg=0;

	//A.D.T Variables
	dbFED_SetValue = HIGH_FREQ_PWM_DEADBAND_FED_COUNT;

	dbRED_SetValue = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
	dbRED_SetValue_Temp = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;

	dbRED_SetValue1 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
	dbRED_SetValue_Temp1 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;

	dbRED_SetValue2 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
	dbRED_SetValue_Temp2 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;

	dbRED_SetValue3 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
	dbRED_SetValue_Temp3 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;

	dbRED_Min=HIGH_FREQ_PWM_DEADBAND_RED_MIN_COUNT;
	dbRED_Max=HIGH_FREQ_PWM_DEADBAND_RED_MAX_COUNT;


	//Variable initialization
	iL1_sensed=0;
	iL1_senseOffset=0;

	iL2_sensed=0;
	iL2_senseOffset=0;

	iL3_sensed=0;
	iL3_senseOffset=0;

	ac_cur_sensed=0;
	ac_cur_senseOffset=0;

	vBus_sensed=0;
	vBus_sensedOffset=0;
	vBusRef=VBUS_TYPICAL/VDCBUS_MAX_SENSE;

	ac_cur_ref=0.03;

	ac_L_sensed=0;
	ac_L_senseOffset=0;

	ac_N_sensed=0;
	ac_N_senseOffset=0;

	ac_vol_senseOffset=0;

	dutyPU=0;
	dutyPU_DC=0;
	dutyPWMREG=0;
	halfPWMPeriod=0;

	closeGiLoop=0;
	closeGvLoop=0;
	clearTrip=0;
	rlyConnect=0;
	firstTimeGvLoop=1;
	acSine=0;
	acSinePrev=0;
	zeroCrossDetectFlag=0;
	ac_cur_sensed_dc_average=0;

	vBus_sensedFiltered_notch1=0;
	vBus_sensedFiltered_notch2=0;

	threshold_PZC1=PWM_SWITCHING_PZC1;
	threshold_PZC2=PWM_SWITCHING_PZC2;
	threshold_NZC1=PWM_SWITCHING_NZC1;
	threshold_NZC2=PWM_SWITCHING_NZC2;


    autoStartSlew=0;

    vRmsAvg=0;
    vBusAvg=0;

	dutyPU_DC=0.5f;

	thetaOffset=0;

	pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;

//	Interleaving variable and state Initilazation
	iref1=PHASE_SHEDDING_1PH_2PH_TRANSITION_CURRENT/I_MAX_SENSE;
	iref2=PHASE_SHEDDING_2PH_3PH_TRANSITION_CURRENT/I_MAX_SENSE;
	deltaIref=PHASE_SHEDDING_TRANSITION_CURRENT_HYSTERESIS/I_MAX_SENSE;
	interleaving_State.enum_interleavingState=interleavingState_3ph;

	nonLinearVoltageLoopFlagDelay=0;
	nonLinearVoltageLoopFlag=0;

}


void computeNotchFltrCoeff(DCL_DF22 *coeff, float32_t Fs, float32_t notch_freq,
                           float32_t c1, float32_t c2)
{
	float32_t temp1;
	float32_t temp2;
	float32_t wn2;
	float32_t Ts;
	Ts=1/Fs;

	// pre warp the notch frequency
	wn2=2*Fs*tanf(notch_freq* PI_VALUE*Ts);

	temp1= 4*Fs*Fs + 4* wn2 * c2 * Fs + wn2*wn2;
	temp2= 1/ ( 4*Fs*Fs + 4* wn2 * c1 * Fs + wn2*wn2);

	coeff->b0 = temp1* temp2;
	coeff->b1 = (-8*Fs*Fs + 2* wn2* wn2)* temp2;
	coeff->b2 = (4*Fs*Fs-4*wn2*c2*Fs+wn2*wn2)*temp2;
	coeff->a1 = (-8*Fs*Fs + 2* wn2* wn2)*temp2;
	coeff->a2 = (4*Fs*Fs-4*wn2*c1*Fs+wn2*wn2)*temp2;


}
//TODO updateBoardStatus()
void updateBoardStatus()
{
	if(board_Status.enum_boardStatus==boardStatus_NoFault)
	{
		if((PWM_TRIP_STATUS(EPWM1_BASE)&EPWM_TZ_INTERRUPT_CBC)!=0 )
		    board_Status.enum_boardStatus=boardStatus_EmulatorStopTrip;
	}

}




//===========================================================================
// No more.
//===========================================================================
