/* ============================================================================
System Name:

File Name:      pfc1ph3ilttpl.h

Target:

Author:

Description:
===========================================================================  */
//#############################################################################
// $TI Release: TIDM_1007 v1.00.00.00 $
// $Release Date: Thu May 28 13:35:58 CDT 2020 $
// $Copyright:
// Copyright (C) 2020 Texas Instruments Incorporated - http://www.ti.com/
//
// ALL RIGHTS RESERVED
// $
//#############################################################################

#ifndef PFC1PH3ILTTPL_H
#define PFC1PH3ILTTPL_H

#ifdef __cplusplus

extern "C" {
#endif

//*****************************************************************************
// the includes
//*****************************************************************************
#include <stdint.h>
#include <stddef.h>

#ifndef __TMS320C28XX_CLA__
#include <math.h>
#else
#include <CLAmath.h>
#endif

// this is the file generated by the powerSUITE page for this system
#include "pfc1ph3ilttpl_settings.h"
#include "pfc1ph3ilttpl_board.h"

// Library header files
// DCL Library is used for the controller implementation
// GI -> series form of DCL_PI for current loop (gi),
// GV -> parallel form of DCL_PI for voltage loop (gv),
// NOTCH_2P2Z -> notch filter using DCL_DF22 which implements a 2p2z structure
#ifndef __TMS320C28XX_CLA__
#include "DCLF32.h"
#define GI_STRUCT DCL_PI
// GI on C28x ,run DCL_PI routine written in assembly
#define GI_RUN DCL_runPI_C1
#define GV_STRUCT DCL_PI
// gv on C28x, run DCL_PI routine written in assembly
#define GV_RUN DCL_runPI_C4
#define NOTCH_2P2Z_STRUCT DCL_DF22
// notch filter on C28x, run DCL_DF22 routine written in assembly
#define NOTCH_2P2Z_RUN DCL_runDF22_C1
#else
#include "DCLCLA.h"
//GI
#define GI_STRUCT DCL_PI_CLA
// GI on CLA, run DCL_PI routine written in CLA assembly
#define GI_RUN DCL_runPI_L1

#define GV_STRUCT DCL_PI_CLA
// gv on CLA, run DCL_PI routine written as inline function
// because it is called in the background task
#define GV_RUN DCL_runPI_L4

#define NOTCH_2P2Z_STRUCT DCL_DF22_CLA
// notch filter on CLA, run DCL_DF22 routine written as inline function
// because it is called in the background task
#define NOTCH_2P2Z_RUN DCL_runDF22_L4

#pragma FUNC_ALWAYS_INLINE(DCL_runDF22_L4)
#pragma FUNC_ALWAYS_INLINE(CLAsqrt_inline)

#define PR_CNTL_STRUCT DCL_DF22_CLA

#endif

#include "dlog_4ch.h"
#include "rampgen.h"
#include "spll_1ph_sogi.h"
#include "spll_1ph_notch.h"
#include "spll_1ph_sogi_fll.h"
#include "power_meas_sine_analyzer.h"
#pragma FUNC_ALWAYS_INLINE(POWER_MEAS_SINE_ANALYZER_run)
#pragma FUNC_ALWAYS_INLINE(EPWM_setActionQualifierContSWForceAction)


#ifndef __TMS320C28XX_CLA__
#include "sfra_f32.h"
#include "sfra_gui_scicomms_driverlib.h"
#else
#define SFRA_F32_inject(m)    m
#define SFRA_F32_collect(m,n)
#endif

//*****************************************************************************
// defines
//*****************************************************************************

#define PWM_SWITCHING_NZC1 0.01f
#define PWM_SWITCHING_NZC2 -0.01f

#define PWM_SWITCHING_PZC1 -0.01f
#define PWM_SWITCHING_PZC2 0.01f

#define EMAVG_MACRO(in, out, multiplier) out = ((in - out)*multiplier) + out;

//=============================================================================
// System Settings
//-----------------------------------------------------------------------------

//Add any system specific setting below
#if SFRA_TYPE==SFRA_VOLTAGE // voltage loop

#define SFRA_FREQ_START 2
#define SFRA_FREQ_LENGTH 30
//SFRA step Multiply = 10^(1/No of steps per decade(40))
#define SFRA_FREQ_STEP_MULTIPLY (float32_t)1.08
#define SFRA_AMPLITUDE (float32_t)0.005

#elif SFRA_TYPE==SFRA_CURRENT // current loop

#if DC_CHECK==1
#define SFRA_FREQ_START 30
//SFRA step Multiply = 10^(1/No of steps per decade(40))
#define SFRA_FREQ_STEP_MULTIPLY (float32_t)1.07
#define SFRA_AMPLITUDE (float32_t)0.005
#else
#define SFRA_FREQ_START 200
//SFRA step Multiply = 10^(1/No of steps per decade(40))
#define SFRA_FREQ_STEP_MULTIPLY (float32_t)1.052
#define SFRA_AMPLITUDE (float32_t)0.01
#endif

#define SFRA_FREQ_LENGTH 100

#endif

#define GRID_MAX_FREQ 70
#define GRID_MIN_FREQ 40

#define MATH_EMAVG2_F_MACRO(In, Out, Mul)   \
    Out = (_IQmpy((In - Out), Mul) + Out);  \


//*****************************************************************************
// typedefs
//*****************************************************************************

typedef union{
    enum
    {
        BuildLevel1_OpenLoop_AC = 0,
        BuildLevel1_OpenLoop_DC=1,
        BuildLevel2_CurrentLoop_AC = 2,
        BuildLevel2_CurrentLoop_DC = 3,
        BuildLevel3_VoltageLoop_AC = 4,
        BuildLevel3_VoltageLoop_DC = 5,
        BuildLevel1_OpenLoop_AC_CLA = 6,
        BuildLevel1_OpenLoop_DC_CLA = 7,
        BuildLevel2_CurrentLoop_AC_CLA = 8,
        BuildLevel2_CurrentLoop_DC_CLA = 9,
        BuildLevel3_VoltageLoop_AC_CLA = 10,
        BuildLevel3_VoltageLoop_DC_CLA = 11,
        BuildLevel4_OpenLoop_AC = 12,
        BuildLevel4_OpenLoop_DC=13,
        BuildLevel5_CurrentLoop_AC_ResLoad = 14,
        BuildLevel5_CurrentLoop_DC = 15,
        BuildLevel6_CurrentLoop_GridConnected = 16,
        BuildLevel6_CurrentLoop_DC = 17,
        BuildLevel4_OpenLoop_AC_CLA = 18,
        BuildLevel4_OpenLoop_DC_CLA = 19,
        BuildLevel5_CurrentLoop_AC_ResLoad_CLA = 20,
        BuildLevel5_CurrentLoop_DC_CLA = 21,
        BuildLevel6_CurrentLoop_AC_GridConnected_CLA = 22,
        BuildLevel6_CurrentLoop_DC_CLA = 23,
    }enum_buildLevel ;
    int32_t pad;
}buildLevel;

extern volatile buildLevel build_Level;

typedef union{
    enum
    {
        boardState_OFF = 0,
        boardState_CheckACInput = 1,
        boardState_RlyConnect = 2,
        boardState_PowerStageON=3,
        boardState_TripCondition = 4,
    }enum_boardState;
    int32_t pad;
}boardState;

extern volatile boardState board_State;

typedef union{
    enum
    {
        boardStatus_Idle = 0,
        boardStatus_NoFault=1,
        boardStatus_OverCurrentTrip = 2,
        boardStatus_EmulatorStopTrip = 3,
        boardStatus_OverVoltageTrip = 4,
        boardStatus_InputUnderVoltageTrip = 5,
        boardStatus_NoGridConnection = 6,
    }enum_boardStatus;
    int32_t pad;
}boardStatus;

extern volatile boardStatus board_Status;

typedef union{
    enum
    {
        pwmSwState_normalOperation = 0,
        pwmSwState_positiveHalf = 1,
        pwmSwState_negativeZeroCrossing1=2,
        pwmSwState_negativeZeroCrossing2=3,
        pwmSwState_negativeZeroCrossing3=4,
        pwmSwState_negativeHalf = 5,
        pwmSwState_positiveZeroCrossing1=6,
        pwmSwState_positiveZeroCrossing2=7,
        pwmSwState_positiveZeroCrossing3=8,
        pwmSwState_defaultState=9,
    }enum_pwmSwState;
    int32_t pad;
}pwmSwState;

extern volatile pwmSwState pwm_SwState;

typedef union{
    enum
    {
        interleavingState_1ph = 0,
        interleavingState_2ph = 1,
        interleavingState_3ph = 2,
    }enum_interleavingState;
    int32_t pad;
}interleavingState;

extern volatile interleavingState interleaving_State;

//*****************************************************************************
// globals
//*****************************************************************************

// RAMP to generate forced angle when grid is not present
extern RAMPGEN rgen1;
extern volatile float32_t sineAngle;

// current controller
extern GI_STRUCT gi;

// voltage controller
extern GV_STRUCT gv;

extern SPLL_1PH_SOGI spll1;
extern SPLL_1PH_NOTCH spll2;
extern SPLL_1PH_SOGI_FLL spll3;

extern NOTCH_2P2Z_STRUCT notch1, notch2;

extern NOTCH_2P2Z_STRUCT notch_50Hz, notch_60Hz;
extern NOTCH_2P2Z_STRUCT spll_notch_50Hz, spll_notch_60Hz;

extern uint16_t guiPowerStageStart;
extern uint16_t guiPowerStageStop;

#if SFRA_TYPE!=SFRA_DISABLED
extern float32_t plantMagVect[SFRA_FREQ_LENGTH];
extern float32_t plantPhaseVect[SFRA_FREQ_LENGTH];
extern float32_t olMagVect[SFRA_FREQ_LENGTH];
extern float32_t olPhaseVect[SFRA_FREQ_LENGTH];
extern float32_t freqVect[SFRA_FREQ_LENGTH];
#endif

//TODO  Control Variables
extern float32_t iL1_sensed, iL2_sensed, iL3_sensed;
extern float32_t iL1_senseOffset, iL2_senseOffset, iL3_senseOffset;
extern float32_t iL1_sensed_scaled, iL2_sensed_scaled, iL3_sensed_scaled;
// AC Current Measurement
extern float32_t ac_cur_sensed;
extern float32_t ac_cur_sensed_scaled;
extern float32_t ac_cur_senseOffset;
extern float32_t ac_cur_sensed_dc_average;

extern volatile float32_t thetaOffset;
// Output Voltage measurement
extern volatile float32_t vBus_sensed;
extern volatile float32_t vBus_sensedOffset;

//Input voltage measurement
extern volatile float32_t ac_L_sensed, ac_N_sensed, ac_vol_sensed;
extern volatile float32_t ac_L_senseOffset, ac_N_senseOffset, ac_vol_senseOffset;

// Filtered DC bus measurement
extern volatile float32_t vBusAvg;

extern volatile float32_t vRmsAvg;

extern uint32_t dbRED_SetValue;
extern uint32_t dbRED_SetValue_Temp;
extern uint32_t dbFED_SetValue;
extern uint32_t dbRED_SetValue1;
extern uint32_t dbRED_SetValue_Temp1;
extern uint32_t dbRED_SetValue2;
extern uint32_t dbRED_SetValue_Temp2;
extern uint32_t dbRED_SetValue3;
extern uint32_t dbRED_SetValue_Temp3;
extern uint32_t dbRED_Min;
extern uint32_t dbRED_Max;

extern volatile float32_t softstart_duty;
// Reference variables

//  current set point
extern volatile float32_t ac_cur_ref;
extern float32_t ac_cur_ref_inst;
extern float32_t ac_cur_ref_inst_prev;

extern volatile float32_t inductor_voltage_drop_feedforward;
// voltage set point
extern volatile float32_t vBusRef, vBusRefSlewed;

// Display Values
extern volatile float32_t guiVbus, guiVbusMax;
extern volatile float32_t guiVbus_threshold;
extern volatile float32_t guiACFreq;
extern volatile float32_t guiPrms;
extern volatile float32_t guiIrms;
extern volatile float32_t guiVrms, guiVrmsMax;
extern volatile float32_t guiVrms_threshold;
extern volatile float32_t guiVavg;
extern volatile float32_t guiVin;
extern volatile float32_t guiIi;
extern volatile float32_t guiIo;
extern volatile float32_t guiPowerFactor;
extern volatile float32_t guiVA;
extern volatile float32_t guiFreqAvg;
extern volatile float32_t guiVema;

extern volatile float32_t guiVrmsEMAvg;

extern volatile int32_t autoStartSlew;

// Phase-Shedding Variables
extern volatile float32_t iref1;
extern volatile float32_t iref2;
extern volatile float32_t deltaIref;


// Variables used to calibrate measurement offsets
//Offset filter coefficient K1: 0.05/(T+0.05);
extern float32_t k1;
//Offset filter coefficient K2: T/(T+0.05)
extern float32_t k2;
extern int16_t offsetCalCounter;
extern float32_t offset165;

// Duty variables for inverter (per unit and compare register values)
// per unit duty value
extern volatile float32_t dutyPU, dutyPU_DC;
// duty value written into the PWM register
extern volatile int32_t dutyPWMREG,dutyPWMREGCompl;
// PWM TBPRD Half
extern volatile float32_t halfPWMPeriod;

// Flags for clearing trips and closing the loops and the Relay
extern volatile int32_t closeGiLoop,closeGvLoop, clearTrip, rlyConnect,
                        firstTimeGvLoop;

// Flags for detecting ZCD
extern volatile float32_t acSine, acSinePrev;
extern volatile float32_t acCosine, acCosinePrev;
extern volatile int32_t zeroCrossDetectFlag;

extern volatile int32_t state_slew;
extern volatile int32_t state_slew_max;

extern volatile int32_t tempChange;
extern volatile int32_t period;

extern volatile float32_t gi_out,gv_out;

extern volatile float32_t phase1ScaleFactor;
extern volatile float32_t phase2ScaleFactor;
extern volatile float32_t phase3ScaleFactor;

// Sine analyzer block for RMS Volt, Curr and Power measurements
extern POWER_MEAS_SINE_ANALYZER sine_mains;

#ifndef __TMS320C28XX_CLA__
// datalogger
extern DLOG_4CH dLog1;
extern float32_t dBuff1[100], dBuff2[100], dBuff3[100], dBuff4[100];
extern float32_t dVal1, dVal2, dVal3, dVal4;

#endif

extern volatile float32_t vBusSensedBuff[10];
extern volatile float32_t vBus_sensedFiltered;
extern volatile int32_t vBusSensedBuffIndex;

extern volatile float32_t vBus_sensedFiltered_notch1,vBus_sensedFiltered_notch2;

extern volatile float32_t threshold_PZC1,threshold_PZC2,threshold_NZC1,
                      threshold_NZC2;

extern volatile float32_t gi_out_prev;

extern volatile float32_t ac_vol_sensed_buff[5];
extern volatile int32_t ac_vol_sensed_buff_index;
extern volatile int32_t ac_sign_filtered;

extern volatile float32_t spll_sine,spll_cosine;
extern volatile float32_t voltage_error;

// 1 if using non linear voltage loop, 0 otherwise
extern volatile uint32_t nonLinearVoltageLoopFlag;

extern volatile uint32_t softStartDeadBandFED;

//*****************************************************************************
// the function prototypes
//*****************************************************************************

#ifndef __TMS320C28XX_CLA__
void setupSFRA();
void globalVariablesInit();
#else
#endif

inline void detectZeroCrossingClearPWMTrip(uint32_t base1, uint32_t base2,
                                           uint32_t base3, uint32_t base4);
void computeNotchFltrCoeff(NOTCH_2P2Z_STRUCT *coeff, float32_t Fs, float32_t notch_freq,
                           float32_t c1, float32_t c2);

#pragma FUNC_ALWAYS_INLINE(read_AC_Curr_Voltage)
static inline void read_AC_Curr_Voltage()
{
    ac_cur_sensed =  (((float32_t)(AC_CUR_FB +  AC_CUR_FB_2 +  AC_CUR_FB_3))
                        *0.333*ADC_PU_SCALE_FACTOR
                        - (ac_cur_senseOffset))*-2.0;

    ac_L_sensed = (((float32_t) (AC_L_FB+AC_L_FB_2+AC_L_FB_3 )
                        *0.333* ADC_PU_SCALE_FACTOR));
    ac_N_sensed = (((float32_t) (AC_N_FB+AC_N_FB_2+AC_N_FB_3 )
                        *0.333* ADC_PU_SCALE_FACTOR));
}

#pragma FUNC_ALWAYS_INLINE(read_CurrIndividual_BusVoltage)
static inline void read_CurrIndividual_BusVoltage()
{
    iL1_sensed = (((float32_t) (IL1_FB )* ADC_PU_SCALE_FACTOR -
            iL1_senseOffset))*2.0;
    iL2_sensed = (((float32_t) (IL2_FB )* ADC_PU_SCALE_FACTOR -
            iL2_senseOffset))*2.0;
    iL3_sensed = (((float32_t) (IL3_FB )* ADC_PU_SCALE_FACTOR -
            iL3_senseOffset))*2.0;

    vBus_sensed = ((float32_t)(VBUS_FB  + VBUS_FB_2 + VBUS_FB_3 )*
                        ADC_PU_SCALE_FACTOR*(1.0/3.0));
}

#pragma FUNC_ALWAYS_INLINE(pfcControlCode)
//TODO control Code
static inline void pfcControlCode(void)
{
    setProfilingGPIO();

    // Read Current and Voltage Measurements
    read_AC_Curr_Voltage();
    ac_vol_sensed = ac_L_sensed - ac_N_sensed - ac_vol_senseOffset;

    //TODO BUILD 1 and 4
    //------------------------------------------------------------------------
    #if (INCR_BUILD == 1 || INCR_BUILD == 4 ) // Open Loop Check
    //------------------------------------------------------------------------

    // Generate the RAMP Angle
    RAMPGEN_run(&rgen1);

    // Use the angle value to compute the sine value
    acSinePrev = acSine;
    acSine =sinf(rgen1.out * ((float32_t)6.283185307));

    detectZeroCrossingClearPWMTrip(LOW_FREQ_PWM_BASE,HIGH_FREQ_PWM1_BASE,
                                   HIGH_FREQ_PWM2_BASE, HIGH_FREQ_PWM3_BASE);

		#if DC_CHECK==1
			dutyPU=dutyPU_DC;
		#else
			dutyPU = dutyPU_DC * acSine;
		#endif


    #endif // (INCR_BUILD == 1)

    //TODO BUILD 2, 3, 4, 5, and 6
    //-------------------------------------------------------------------------
    #if (INCR_BUILD == 2 || INCR_BUILD==3 || INCR_BUILD==5 || INCR_BUILD==6)  //Closed Current Loop
    //-------------------------------------------------------------------------

        #if INCR_BUILD==5
        //    For Build Level 5 Inverter Resistive Load Test
        RAMPGEN_run(&rgen1);
        acSinePrev = acSine;
        acSine =sinf(rgen1.out * ((float)6.283185307));
        acCosine =cosf(rgen1.out * ((float)6.283185307));
        #endif

        if(closeGiLoop==1)
        {

        #if SFRA_TYPE == SFRA_CURRENT
            gi_out=GI_RUN(&gi, ac_cur_sensed,
                                SFRA_F32_inject(ac_cur_ref_inst));

        #else
            gi_out=GI_RUN(&gi, ac_cur_sensed, ac_cur_ref_inst);
        #endif


        dutyPU=( gi_out+(ac_vol_sensed*
                        (float32_t)VAC_MAX_SENSE/(float32_t)VDCBUS_MAX_SENSE)
                       -inductor_voltage_drop_feedforward )/
                 (vBus_sensed);



        dutyPU= (dutyPU>(float32_t)(1.0f))?(float32_t)(1.0f):dutyPU;
        dutyPU= (dutyPU<(float32_t)(-1.0f))?(float32_t)(-1.0f):dutyPU;

        }
        else
        {
            dutyPU=0.01f;
        }

    #endif


    //-------------------------------------------------------------------------
    // PWM Driver
    //-------------------------------------------------------------------------
#pragma diag_suppress=770
#pragma diag_suppress=173
#if DC_CHECK ==0
    switch(pwm_SwState.enum_pwmSwState)
    {
        case pwmSwState_normalOperation:

            // wait for positive zero crossing i.e. state 6
            if(acSine < threshold_PZC2  && acSinePrev<0)
            {
                pwm_SwState.enum_pwmSwState=pwmSwState_positiveZeroCrossing1;

                dutyPU=0.004;

                AQ_SW_FORCE_PWMxA_LOW_PWMxB_LOW(LOW_FREQ_PWM_BASE);

                SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);

                SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);

                SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
                SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);

                gi.i10=0;
                gi.i6=1;
                gi_out=0;

                clearPWMTripFlags(LOW_FREQ_PWM_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM1_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM2_BASE);
                clearPWMTripFlags(HIGH_FREQ_PWM3_BASE);

                ac_cur_ref_inst=0.00f;
                ac_cur_ref_inst=0.00f;
                closeGiLoop=1;
                closeGvLoop=1;

            }

            break;

        case pwmSwState_positiveHalf:

        #if INCR_BUILD <= 3
            //AQCSFRC xA LOW, xB HIGH
            AQ_SW_FORCE_PWMxA_LOW_PWMxB_HIGH(LOW_FREQ_PWM_BASE);
        #endif

            SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,dbRED_SetValue);
            SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,dbRED_SetValue);
            SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,dbRED_SetValue);

            state_slew++;

            if ( acSine < threshold_NZC1   || ac_sign_filtered == 0 )
                {
                    pwm_SwState.enum_pwmSwState=pwmSwState_negativeZeroCrossing1;

                    //AQCSFRC xA LOW, xB LOW
                    AQ_SW_FORCE_PWMxA_LOW_PWMxB_LOW(LOW_FREQ_PWM_BASE);
                }


            break;

        case pwmSwState_negativeZeroCrossing1 :
            state_slew++;


            SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);

            SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);

            SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);

            dutyPU = 0.004f;

            // now wait till actual zero in this state before switching to zero2
            if(acSine <= 0 )
            {
                pwm_SwState.enum_pwmSwState=pwmSwState_negativeZeroCrossing2;
            }
            break;

        case pwmSwState_negativeZeroCrossing2 :
            state_slew++;

            dutyPU = 0.004f;
            // wait till certain positive threshold before moving to the normal operation

            #if INCR_BUILD > 3
                //AQCSFRC xA HIGH, xB LOW
                //In INV mode, Low_FREQ FET has to be turned on earlier than PFC mode
                AQ_SW_FORCE_PWMxA_HIGH_PWMxB_LOW(LOW_FREQ_PWM_BASE);
            #endif

            if(acSine < threshold_NZC2)
            {
                state_slew=0;
                pwm_SwState.enum_pwmSwState=pwmSwState_negativeZeroCrossing3;
                // Enable xA xB Swap
                ENABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM1_BASE);
                ENABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM2_BASE);
                ENABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM3_BASE);
                softstart_duty=0;

                gi.i10=0;
                gi.i6=1;
                gi.Ki=0;
                gi_out=0;
            }

            break;

        case pwmSwState_negativeZeroCrossing3 :
            // wait till certain positive threshold before moving to the normal operation

            softstart_duty= softstart_duty+1;

            softStartDeadBandFED=PFC_PWM_PERIOD - softstart_duty*50;
            if(softStartDeadBandFED<dbFED_SetValue )
            {
                softStartDeadBandFED=dbFED_SetValue;
            }

            SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE, softStartDeadBandFED );
            SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE, softStartDeadBandFED );
            SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE, softStartDeadBandFED );

            state_slew++;

            if( fabsf(dutyPU) <  0.004f)
            {
                if (dutyPU > 0)
                {                     dutyPU = 0.004;                 }
                else
                {                     dutyPU = -0.004;                 }
            }


            if(state_slew > state_slew_max  && ac_sign_filtered ==0 )
            {
                gi.i10=0;
                gi.i6=1;
                gi.Ki=GI_PI_KI;
                gi.Umax=GI_PI_MAX;
                gi.Umin=GI_PI_MIN;
                gi_out=0;

                pwm_SwState.enum_pwmSwState=pwmSwState_negativeHalf;
                SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE, dbFED_SetValue);
                SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE, dbFED_SetValue);
                SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE, dbFED_SetValue);

                state_slew=0;
            }
            break;

        case pwmSwState_negativeHalf:

            #if INCR_BUILD <= 3
                //AQCSFRC xA HIGH, xB LOW
                AQ_SW_FORCE_PWMxA_HIGH_PWMxB_LOW(LOW_FREQ_PWM_BASE);
            #endif

            SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,dbRED_SetValue1);
            SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,dbRED_SetValue2);
            SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,dbRED_SetValue3);

            if ( acSine > threshold_PZC1 || ac_sign_filtered==1)
            {
                pwm_SwState.enum_pwmSwState=pwmSwState_positiveZeroCrossing1;

                //AQCSFRC xA LOW, xB LOW
                AQ_SW_FORCE_PWMxA_LOW_PWMxB_LOW(LOW_FREQ_PWM_BASE);

            }

            break;

        case pwmSwState_positiveZeroCrossing1 :
            state_slew++;
            SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);

            SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);

            SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);


            dutyPU = 0.004f;

            if(acSine >= 0 )
            {
                pwm_SwState.enum_pwmSwState=pwmSwState_positiveZeroCrossing2;
                softstart_duty=0.0;

            }
            break;

        case pwmSwState_positiveZeroCrossing2 :
            state_slew++;

            dutyPU = 0.004f;

            #if INCR_BUILD > 3
            //AQCSFRC xA LOW, xB HIGH
            //In INV mode, Low_FREQ FET has to be turned on earlier than PFC mode
                AQ_SW_FORCE_PWMxA_LOW_PWMxB_HIGH(LOW_FREQ_PWM_BASE);
            #endif

            if(acSine > threshold_PZC2 )
            {
                pwm_SwState.enum_pwmSwState=pwmSwState_positiveZeroCrossing3;
                state_slew=0;
                softstart_duty=0;

                // Dead band output swap disabled
                DISABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM1_BASE);
                DISABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM2_BASE);
                DISABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM3_BASE);

                gi.i10=0;
                gi.i6=1;
                gi_out=0;
                gi.Ki=0;
            }
            break;

        case pwmSwState_positiveZeroCrossing3 :
            state_slew++;

            if( fabsf(dutyPU) <  0.004f)
            {
                if (dutyPU > 0)
                {                     dutyPU = 0.004;                 }
                else
                {                     dutyPU = -0.004;                 }
            }

            softstart_duty= softstart_duty+1;

            softStartDeadBandFED=PFC_PWM_PERIOD - softstart_duty*50;
            if(softStartDeadBandFED<dbFED_SetValue )
            {
                softStartDeadBandFED=dbFED_SetValue;
            }

            SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE, softStartDeadBandFED );
            SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE, softStartDeadBandFED );
            SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE, softStartDeadBandFED );

            if(state_slew > state_slew_max && ac_sign_filtered==1)
            {
                gi.i10=0;
                gi.i6=1;
                gi_out=0;
                gi.Ki=GI_PI_KI;

                pwm_SwState.enum_pwmSwState=pwmSwState_positiveHalf;

                SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE, dbFED_SetValue);
                SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE, dbFED_SetValue);
                SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE, dbFED_SetValue);

                state_slew=0;
            }

            break;

        case pwmSwState_defaultState:
            dutyPU=0.01f;

            AQ_SW_FORCE_PWMxA_LOW_PWMxB_LOW(LOW_FREQ_PWM_BASE);

            SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,PFC_PWM_PERIOD);

            SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,PFC_PWM_PERIOD);

            SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
            SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,PFC_PWM_PERIOD);
            break;
        default:
            pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
    }

#endif

#if DC_CHECK==1
    if(ac_vol_sensed>0 && dutyPU>0)
    {
        //AQCSFRC xA LOW, xB HIGH
        AQ_SW_FORCE_PWMxA_LOW_PWMxB_HIGH(LOW_FREQ_PWM_BASE);
        
        SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,dbRED_SetValue);
        SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,dbRED_SetValue);
        SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,dbRED_SetValue);

        SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,dbFED_SetValue);
        SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,dbFED_SetValue);
        SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,dbFED_SetValue);

        // Dead band output swap disabled
        DISABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM1_BASE);
        DISABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM2_BASE);
        DISABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM3_BASE);
    }
    else if ((ac_vol_sensed<0 && dutyPU<0))
    {
        //AQCSFRC xA HIGH, xB LOW
        AQ_SW_FORCE_PWMxA_HIGH_PWMxB_LOW(LOW_FREQ_PWM_BASE);

        SET_PWM_DBRED(HIGH_FREQ_PWM1_BASE,dbRED_SetValue);
        SET_PWM_DBRED(HIGH_FREQ_PWM2_BASE,dbRED_SetValue);
        SET_PWM_DBRED(HIGH_FREQ_PWM3_BASE,dbRED_SetValue);

        SET_PWM_DBFED(HIGH_FREQ_PWM1_BASE,dbFED_SetValue);
        SET_PWM_DBFED(HIGH_FREQ_PWM2_BASE,dbFED_SetValue);
        SET_PWM_DBFED(HIGH_FREQ_PWM3_BASE,dbFED_SetValue);

        // Dead band output swap disabled
        ENABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM1_BASE);
        ENABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM2_BASE);
        ENABLE_SWAP_DEADBAND_OUTPUT(HIGH_FREQ_PWM3_BASE);
    }

#endif

    update_pfc1ph3ilttplPWM(LOW_FREQ_PWM_BASE,HIGH_FREQ_PWM1_BASE,
                            HIGH_FREQ_PWM2_BASE, HIGH_FREQ_PWM3_BASE,
                            dutyPU, phase1ScaleFactor,phase2ScaleFactor,
                            phase3ScaleFactor);

#pragma diag_warning=770
#pragma diag_warning=173

#if ADAPTIVE_DEADTIME==1
    ac_cur_sensed_scaled=fabsf(ac_cur_sensed*I_MAX_SENSE);
    ac_cur_sensed_scaled=(ac_cur_sensed_scaled<0.01f)?0.01f:
                                            ac_cur_sensed_scaled;

    dbRED_SetValue_Temp= (uint16_t)(
                            (float32_t)100000000.0*(2.0*VBUS_TYPICAL*GAN_COSS)/
                            (ac_cur_sensed_scaled*0.33)
                            );

    dbRED_SetValue = (dbRED_SetValue_Temp<dbRED_Min)?dbRED_Min:
                                                dbRED_SetValue_Temp;
    dbRED_SetValue = (dbRED_SetValue_Temp>dbRED_Max)?dbRED_Max:
                                                dbRED_SetValue_Temp;

//      Individual Peak Current Sensing
//      Phase A
    iL1_sensed_scaled=fabsf(iL1_sensed*16.5f);
    iL1_sensed_scaled=(iL1_sensed_scaled<0.0033f)?0.0033f:iL1_sensed_scaled;

    dbRED_SetValue_Temp1= (uint16_t)(
                            (float32_t)100000000.0*(2.0*VBUS_TYPICAL*GAN_COSS)/
                            (iL1_sensed_scaled)
                            );
    dbRED_SetValue1 = (dbRED_SetValue_Temp1<dbRED_Min)?dbRED_Min:
                                                    dbRED_SetValue_Temp1;
    dbRED_SetValue1 = (dbRED_SetValue_Temp1>dbRED_Max)?dbRED_Max:
                                                    dbRED_SetValue_Temp1;

//      Phase B
    iL2_sensed_scaled=fabsf(iL2_sensed*16.5f);
    iL2_sensed_scaled=(iL2_sensed_scaled<0.0033f)?0.0033f:iL2_sensed_scaled;

    dbRED_SetValue_Temp2= (uint16_t)(
                             (float32_t)100000000.0*(2.0*VBUS_TYPICAL*GAN_COSS)/
                             (iL2_sensed_scaled)
                             );
    dbRED_SetValue2 = (dbRED_SetValue_Temp2<dbRED_Min)?dbRED_Min:
                                                    dbRED_SetValue_Temp2;
    dbRED_SetValue2 = (dbRED_SetValue_Temp2>dbRED_Max)?dbRED_Max:
                                                    dbRED_SetValue_Temp2;

//      Phase C
    iL3_sensed_scaled=fabsf(iL3_sensed*16.5f);
    iL3_sensed_scaled=(iL3_sensed_scaled<0.0033f)?0.0033f:iL3_sensed_scaled;

    dbRED_SetValue_Temp3= (uint16_t)(
                             (float32_t)100000000.0*(2.0*VBUS_TYPICAL*GAN_COSS)/
                             (iL3_sensed_scaled)
                             );
    dbRED_SetValue3 = (dbRED_SetValue_Temp3<dbRED_Min)?dbRED_Min:
                                                        dbRED_SetValue_Temp3;
    dbRED_SetValue3 = (dbRED_SetValue_Temp3>dbRED_Max)?dbRED_Max:
                                                        dbRED_SetValue_Temp3;

#else
    dbRED_SetValue = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
    dbRED_SetValue1 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
    dbRED_SetValue2 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
    dbRED_SetValue3 = HIGH_FREQ_PWM_DEADBAND_RED_COUNT;
#endif

#if (INCR_BUILD==4 || INCR_BUILD==5)
    //acSine is generated from ramp_gen for resistive load test (no grid synchronization needed)
    if(acSine>0.0)
    {
        ac_sign_filtered=1;
    }
    else
    {
        ac_sign_filtered=0;
    }

        spll_sine=acSine;
        spll_cosine=cosf(rgen1.out * ((float)6.283185307));
#else
    //acSine is obtained from SPLL for grid-connected test
	#if SPLL_METHOD_SELECT==SPLL_1PH_SOGI_SEL
		SPLL_1PH_SOGI_run(&spll1,ac_vol_sensed);
		spll_sine=spll1.sine;
		spll_cosine=spll1.cosine;
		if(fabsf(spll1.ylf[0]) >2.0f)
		{
			if(ac_vol_sensed>0.0)
			{
				ac_sign_filtered=1;
//				setProfilingGPIO();
			}
			else
			{
				ac_sign_filtered=0;
//				resetProfilingGPIO();
			}
		}
		else
		{
			if(spll1.sine>0.0)
			{
				ac_sign_filtered=1;
//				setProfilingGPIO();
			}
			else
			{
				ac_sign_filtered=0;
//                resetProfilingGPIO();
			}
		}

		acSinePrev = acSine;
		acSine = spll_sine;
	#endif


	#if SPLL_METHOD_SELECT==SPLL_1PH_NOTCH_SEL
		SPLL_1PH_NOTCH_run(&spll2,ac_vol_sensed);
		spll_sine=spll2.sine;
		spll_cosine=spll2.cosine;
		if(fabsf(spll2.ylf[0]) >8.0f)
		{

			if(ac_vol_sensed>0.0)
			{
				ac_sign_filtered=1;
			}
			else
			{
				ac_sign_filtered=0;
			}
		}
		else
		{

			if(spll2.sine>0.0)
			{
				ac_sign_filtered=1;
			}
			else
			{
				ac_sign_filtered=0;
			}
		}

		acSinePrev = acSine;
		acSine = spll_sine;
	#endif


	#if SPLL_METHOD_SELECT==SPLL_1PH_SOGI_FLL_SEL
	  SPLL_1PH_SOGI_FLL_run(&spll3,ac_vol_sensed);
	  spll_sine=spll3.sine;
	  spll_cosine=spll3.cosine;

		if(fabsf(spll3.ylf[0]) >0.8f)
		{
	//      acSinePrev = acSine;
	//      acSine=ac_vol_sensed/(sine_mains.vRms*1.414);

			if(ac_vol_sensed>0.0)
			{
				ac_sign_filtered=1;
			}
			else
			{
				ac_sign_filtered=0;
			}
		}
		else
		{

	//      acSinePrev = acSine;
	//      acSine = spll_sine;

			if(spll3.sine>0.0)
			{
				ac_sign_filtered=1;
			}
			else
			{
				ac_sign_filtered=0;
			}
		}

		acSinePrev = acSine;
		acSine = spll_sine;
	#endif
#endif
    //-------------------------------------------------------------------------
    #if (INCR_BUILD == 2 || INCR_BUILD==3 || INCR_BUILD==5 || INCR_BUILD==6 )  //Closed Current Loop
    //-------------------------------------------------------------------------

    if(closeGiLoop==1)
    {
        #if  DC_CHECK ==1
                ac_cur_ref_inst=ac_cur_ref;
        #else
            #if INPUT_CAP_COMPENSATION==1
                ac_cur_ref_inst=ac_cur_ref*spll_sine + thetaOffset*spll_cosine;
            #else
                ac_cur_ref_inst=ac_cur_ref*spll1.sine;
            #endif
        #endif
    }


    #if DC_CHECK ==1
        RAMPGEN_run(&rgen1);
        // Use the angle value to compute the sine value
        acSinePrev = acSine;
        acSine =sinf(rgen1.out * ((float32_t)6.283185307));
    #else

    #endif


    inductor_voltage_drop_feedforward = -(float32_t)1.0*
                        (ac_cur_ref_inst - ac_cur_ref_inst_prev)
                        *( (float32_t)3.0*PFC_INDUCTOR_VALUE*CONTROL_ISR_FREQUENCY*
                                I_MAX_SENSE / VDCBUS_MAX_SENSE);

    ac_cur_ref_inst_prev=ac_cur_ref_inst;

    #endif

#if(SFRA_TYPE == SFRA_CURRENT )
    SFRA_F32_collect((float32_t*)&gi_out,(float32_t*)&ac_cur_sensed);
#endif

    read_CurrIndividual_BusVoltage();

    vBusSensedBuff[vBusSensedBuffIndex++]=vBus_sensed;
    if(vBusSensedBuffIndex>=10)
        vBusSensedBuffIndex=0;

    clearPWMInterruptFlag(C28x_CONTROLISR_INTERRUPT_TRIG_PWM_BASE);
    resetProfilingGPIO();

}

//TODO voltage loop plus instrumentation code
#pragma FUNC_ALWAYS_INLINE(voltageLoopPlusInstrumentationCode)
static inline void voltageLoopPlusInstrumentationCode()
{
    int32_t k=0;
    #if INSTRUMENTATION_ISR_RUNNING_ON == C28x_CORE
        EINT;
    #endif
    setProfilingGPIO2();

    vBus_sensedFiltered=0;

    for(k=0;k<10;k++)
    {
        vBus_sensedFiltered+= vBusSensedBuff[k];
    }

    vBus_sensedFiltered=vBus_sensedFiltered*0.1f;

    vBus_sensedFiltered_notch1=NOTCH_2P2Z_RUN(&notch1,vBus_sensedFiltered);
    vBus_sensedFiltered_notch2=NOTCH_2P2Z_RUN(&notch2,vBus_sensedFiltered_notch1);

    //TODO BUILD 3
    //-------------------------------------------------------------------------
    #if(INCR_BUILD ==3)
    //-------------------------------------------------------------------------

    // Soft start for the voltage set point
    if(closeGvLoop==1)
    {
        if ( fabsf(vBusRef - vBusRefSlewed) > 0.0001f)
        {
            if(vBusRef>vBusRefSlewed)
            {
                vBusRefSlewed = vBusRefSlewed + 0.0001f;
            }
            else
            {
                vBusRefSlewed = vBusRefSlewed - 0.0001f;
            }
        }
        else
        {
            vBusRefSlewed = vBusRef;
        }

        #if NON_LINEAR_VOLTAGE_LOOP==1
            nonLinearVoltageLoopFlag=1;
        #else
            nonLinearVoltageLoopFlag=0;
        #endif

    }

    if(closeGvLoop == 1)
    {
        if(firstTimeGvLoop==1)
        {
            vBusRefSlewed=vBus_sensed;
            firstTimeGvLoop=0;
        }

        voltage_error=fabsf(vBusRefSlewed- vBus_sensedFiltered_notch2);

        if(nonLinearVoltageLoopFlag==0)
        {
            gv.Kp=GV_PI_KP;
            gv.Ki=GV_PI_KI;
        }
        else
        {
            if(voltage_error>0.005f)
            {
                if(gv.Kp<(NON_LINEAR_V_LOOP_KP_MULTIPLIER*GV_PI_KP))
                {
                    gv.Kp=gv.Kp+0.05f;
                }
            }
            else if(voltage_error<0.001f)
            {
                if(gv.Kp>GV_PI_KP)
                {
                    gv.Kp=gv.Kp-0.05f;
                }
            }
        }


        #if SFRA_TYPE == SFRA_VOLTAGE
            gv_out=GV_RUN(&gv, SFRA_F32_inject(vBusRefSlewed),
                                vBus_sensedFiltered_notch2);
        #else
            gv_out=GV_RUN(&gv,vBusRefSlewed,vBus_sensedFiltered_notch2);
        #endif

        #if SFRA_TYPE == SFRA_VOLTAGE
            SFRA_F32_collect((float32_t*)&gv_out,(float32_t*)&vBus_sensedFiltered_notch2);
        #endif

        ac_cur_ref = gv_out * vBusAvg / (vRmsAvg);
    }

    #endif

    EMAVG_MACRO(vBus_sensedFiltered_notch2, vBusAvg, (float32_t)0.000528);

    EMAVG_MACRO((fabsf(ac_vol_sensed)*VAC_MAX_SENSE), guiVrmsEMAvg,
                (float32_t)0.00005);


    // ------------------------------------------------------------------------
    //    Connect inputs of the Datalogger module
    // ------------------------------------------------------------------------

#ifndef __TMS320C28XX_CLA__
    //TODO DLOG

	#if (INCR_BUILD==1 || INCR_BUILD==4)
        // check output voltage and inverter current meas.
        dVal1 = acSine;
        dVal2 = acSine-spll1.sine;
        dVal3 = acSine-spll3.sine;
        dVal4 =ac_cur_sensed;
    #elif (INCR_BUILD==2 || INCR_BUILD==5 || INCR_BUILD==6)
        // check inverter current meas and reference.
        dVal1 = closeGiLoop;
        dVal2 = ac_cur_ref_inst;
        dVal3 = ac_cur_sensed;
        dVal4 = dutyPU;
    #else
        // check inverter voltage meas and reference
        dVal1 = acSine;
        dVal2 = ac_cur_sensed;
        dVal3 = ac_cur_ref;
        dVal4 = spll3.ylf[0];
    #endif
    DLOG_4CH_run(&dLog1);
#endif

#if DC_CHECK==0
    #if INCR_BUILD <= 3
        //Relay connection condition for PFC
        if(guiVrms > 75.0f || guiVbus > 110.0f)
        {
            rlyConnect=1;
        }
        else
        {
            rlyConnect=0;
        }
    #else
        //Relay connection condition for Inverter
        if( guiVbus > guiVbus_threshold)
        {
            rlyConnect=1;
        }
        else
        {
            rlyConnect=0;
        }
    #endif
#endif

    guiVbus=vBus_sensedFiltered_notch2*VDCBUS_MAX_SENSE;
    guiVin=(ac_vol_sensed)*VAC_MAX_SENSE;
    guiIi=ac_cur_sensed*I_MAX_SENSE;

#if  INCR_BUILD <= 3
//    Overvoltage protection for PFC mode
    if(guiVbus>guiVbusMax)
    {
        pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
        forceOSTPWMTrip(HIGH_FREQ_PWM1_BASE);
        forceOSTPWMTrip(HIGH_FREQ_PWM2_BASE);
        forceOSTPWMTrip(HIGH_FREQ_PWM3_BASE);
        EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                EPWM_AQ_OUTPUT_A ,
                                                EPWM_AQ_SW_OUTPUT_LOW);
        EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                EPWM_AQ_OUTPUT_B ,
                                                EPWM_AQ_SW_OUTPUT_LOW);

        if(board_Status.enum_boardStatus==boardStatus_NoFault)
        {
            guiVrmsEMAvg=0;
        }
        board_Status.enum_boardStatus=boardStatus_OverVoltageTrip;
    }
#else
//    Overvoltage protection for Inverter mode
    if(guiVrms>guiVrmsMax)
    {
        pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
        forceOSTPWMTrip(HIGH_FREQ_PWM1_BASE);
        forceOSTPWMTrip(HIGH_FREQ_PWM2_BASE);
        forceOSTPWMTrip(HIGH_FREQ_PWM3_BASE);
        EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                EPWM_AQ_OUTPUT_A ,
                                                EPWM_AQ_SW_OUTPUT_LOW);
        EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                EPWM_AQ_OUTPUT_B ,
                                                EPWM_AQ_SW_OUTPUT_LOW);

        if(board_Status.enum_boardStatus==boardStatus_NoFault)
        {
            guiVrmsEMAvg=0;
        }
        board_Status.enum_boardStatus=boardStatus_OverVoltageTrip;
    }
#endif

#if (DC_CHECK==0)
    #if(INCR_BUILD <= 3)
    //Undervoltage protection for PFC
        if(guiVrms<75.0f)
            {
                pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
                forceOSTPWMTrip(HIGH_FREQ_PWM1_BASE);
                forceOSTPWMTrip(HIGH_FREQ_PWM2_BASE);
                forceOSTPWMTrip(HIGH_FREQ_PWM3_BASE);
                EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                        EPWM_AQ_OUTPUT_A ,
                                                        EPWM_AQ_SW_OUTPUT_LOW);
                EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                        EPWM_AQ_OUTPUT_B ,
                                                        EPWM_AQ_SW_OUTPUT_LOW);

                if(board_Status.enum_boardStatus==boardStatus_NoFault)
                {
                    guiVrmsEMAvg=0;
                }
                board_Status.enum_boardStatus=boardStatus_InputUnderVoltageTrip;
            }

    #else
    //Undervoltage protection for Inverter.
        if(guiVbus<guiVbus_threshold)
            {
                pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;
                forceOSTPWMTrip(HIGH_FREQ_PWM1_BASE);
                forceOSTPWMTrip(HIGH_FREQ_PWM2_BASE);
                forceOSTPWMTrip(HIGH_FREQ_PWM3_BASE);
                EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                        EPWM_AQ_OUTPUT_A ,
                                                        EPWM_AQ_SW_OUTPUT_LOW);
                EPWM_setActionQualifierContSWForceAction(LOW_FREQ_PWM_BASE,
                                                        EPWM_AQ_OUTPUT_B ,
                                                        EPWM_AQ_SW_OUTPUT_LOW);

                if(board_Status.enum_boardStatus==boardStatus_NoFault)
                {
                    guiVrmsEMAvg=0;
                }
                board_Status.enum_boardStatus=boardStatus_InputUnderVoltageTrip;
            }
        // Waiting for the grid connection after applying enough dc bus voltage (for Incremental Build Level 6)
        else if(guiVbus>=340 && autoStartSlew==0 && board_Status.enum_boardStatus==boardStatus_InputUnderVoltageTrip)
            board_Status.enum_boardStatus=boardStatus_NoGridConnection;
    #endif
#endif

    if((PWM_TRIP_STATUS(LOW_FREQ_PWM_BASE)&EPWM_TZ_INTERRUPT_DCAEVT1) != 0)
    {
        pwm_SwState.enum_pwmSwState=pwmSwState_defaultState;

        if(board_Status.enum_boardStatus==boardStatus_NoFault)
        {
            guiVrmsEMAvg=0;
        }

        board_Status.enum_boardStatus=boardStatus_OverCurrentTrip;

//        todo 0722
/*        EPWM_clearTripZoneFlag(LOW_FREQ_PWM_BASE,
                                   (EPWM_TZ_INTERRUPT_OST|EPWM_TZ_INTERRUPT_DCAEVT1));
            EPWM_clearTripZoneFlag(HIGH_FREQ_PWM1_BASE,
                                   (EPWM_TZ_INTERRUPT_OST|EPWM_TZ_INTERRUPT_DCAEVT1));
            EPWM_clearTripZoneFlag(HIGH_FREQ_PWM2_BASE,
                                   (EPWM_TZ_INTERRUPT_OST|EPWM_TZ_INTERRUPT_DCAEVT1));
            EPWM_clearTripZoneFlag(HIGH_FREQ_PWM3_BASE,
                                   (EPWM_TZ_INTERRUPT_OST|EPWM_TZ_INTERRUPT_DCAEVT1));*/
    }

    //Calculate RMS input voltage and input frequency

    sine_mains.i = ac_cur_sensed;
    sine_mains.v = ac_vol_sensed;
    POWER_MEAS_SINE_ANALYZER_run(&sine_mains);
    guiIrms=sine_mains.iRms*I_MAX_SENSE;
    guiVrms=sine_mains.vRms*VAC_MAX_SENSE;
    guiPrms=sine_mains.pRms*VAC_MAX_SENSE*I_MAX_SENSE;
    guiPowerFactor=sine_mains.powerFactor;
    guiVA=sine_mains.vaRms*VAC_MAX_SENSE*I_MAX_SENSE;
    guiFreqAvg = sine_mains.acFreqAvg;
    guiVema=sine_mains.vEma*VAC_MAX_SENSE;

    EMAVG_MACRO(sine_mains.vRms, vRmsAvg, (float32_t)0.000528);

#if DC_CHECK==0
    if(fabsf(acSine)<0.05f)
    {
        if(rlyConnect==1)
        {
            closeRelay();
        }
    }

    if (rlyConnect==0)
    {
        openRelay();
    }
#else
    if(rlyConnect==1)
    {
        closeRelay();
    }
    else
    {
        openRelay();
    }
#endif


#if PHASE_SHEDDING_ENABLED==1
//  Phase shedding
    switch(interleaving_State.enum_interleavingState)
    {
    case interleavingState_3ph:
        if(ac_cur_ref<iref2-deltaIref)
        {
            interleaving_State.enum_interleavingState=interleavingState_2ph;
            //  180 Degree Phase Shift for 2 phase interleaving
            EPWM_setPhaseShift(HIGH_FREQ_PWM2_BASE,
                           ((uint16_t)((float32_t)PFC_PWM_PERIOD/(float32_t)(2.0)))-2);
            EPWM_setCountModeAfterSync(HIGH_FREQ_PWM2_BASE,
                                 EPWM_COUNT_MODE_DOWN_AFTER_SYNC);

            //  Set High FREQ PWM 2 On and Set High FREQ PWM 3 Off
            GPIO_setPinConfig(HIGH_FREQ_PWM2_H_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM2_L_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_H_PHASE_SHEDDING_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_L_PHASE_SHEDDING_GPIO_PIN_CONFIG);
        }

        break;

    case interleavingState_2ph:
        if(ac_cur_ref>iref2+deltaIref)
        {
            interleaving_State.enum_interleavingState=interleavingState_3ph;
            // 120 Degree Phase Shift for 2 phase interleaving
            EPWM_setPhaseShift(HIGH_FREQ_PWM2_BASE,
                           (2+(uint16_t)((float32_t)PFC_PWM_PERIOD/(float32_t)(3.0))));
            EPWM_setCountModeAfterSync(HIGH_FREQ_PWM2_BASE,
                                       EPWM_COUNT_MODE_UP_AFTER_SYNC);

            // Set High FREQ PWM 2 and 3 On
            GPIO_setPinConfig(HIGH_FREQ_PWM2_H_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM2_L_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_H_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_L_GPIO_PIN_CONFIG);
        }

        else if (ac_cur_ref<iref1-deltaIref)
        {
            interleaving_State.enum_interleavingState=interleavingState_1ph;
            //  Set High FREQ PWM2 and 3 Off
            GPIO_setPinConfig(HIGH_FREQ_PWM2_H_PHASE_SHEDDING_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM2_L_PHASE_SHEDDING_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_H_PHASE_SHEDDING_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_L_PHASE_SHEDDING_GPIO_PIN_CONFIG);
        }

        break;

    case interleavingState_1ph:

        if(ac_cur_ref>iref1+deltaIref)
        {
            interleaving_State.enum_interleavingState=interleavingState_2ph;

            //  180 Degree Phase Shift for 2 phase interleaving
            EPWM_setPhaseShift(HIGH_FREQ_PWM2_BASE,
                          ((uint16_t)((float32_t)PFC_PWM_PERIOD/(float32_t)(2.0)))-2);
            EPWM_setCountModeAfterSync(HIGH_FREQ_PWM2_BASE,
                                      EPWM_COUNT_MODE_DOWN_AFTER_SYNC);

            //  Set High FREQ PWM 2 On and Set High FREQ PWM 3 Off
            GPIO_setPinConfig(HIGH_FREQ_PWM2_H_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM2_L_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_H_PHASE_SHEDDING_GPIO_PIN_CONFIG);
            GPIO_setPinConfig(HIGH_FREQ_PWM3_L_PHASE_SHEDDING_GPIO_PIN_CONFIG);
        }

       break;
    }
#else
    interleaving_State.enum_interleavingState=interleavingState_3ph;
#endif


    resetProfilingGPIO2();
}

//TODO detectZeroCrossingClearPWMTrip()
inline void detectZeroCrossingClearPWMTrip(uint32_t base1, uint32_t base2,
                                           uint32_t base3, uint32_t base4)
{
    if((acSinePrev<=(float32_t)(0.00))&&(acSine>(float32_t)(0.00)))
    {
        zeroCrossDetectFlag=1;
    }
    else
    {
        zeroCrossDetectFlag=0;
    }

    if (clearTrip==1 && zeroCrossDetectFlag==1)
    {
        pwm_SwState.enum_pwmSwState=pwmSwState_normalOperation;
        board_Status.enum_boardStatus=boardStatus_NoFault;
        clearTrip=0;
    }

}


#ifdef __cplusplus
}
#endif                                  /* extern "C" */


#endif
