/* ============================================================================
System Name:

File Name:      <BoardName>_board.h

Target:         TIDM-1007, F28004x

Author:

Description:    This file consists of common variables and functions
                for a particular hardware board, like functions to read current
                and voltage signals on the board and functions to setup the
                basic peripherals of the board
                This file must be settings independent, all settings dependent
                code should reside in the parent solution project.
===========================================================================  */
//#############################################################################
// $TI Release: TIDM_1007 v1.00.00.00 $
// $Release Date: Thu May 28 13:35:58 CDT 2020 $
// $Copyright:
// Copyright (C) 2020 Texas Instruments Incorporated - http://www.ti.com/
//
// ALL RIGHTS RESERVED
// $
//#############################################################################

#ifndef PFC1PH3ILTTPL_BOARD_H
#define PFC1PH3ILTTPL_BOARD_H

#ifdef __cplusplus

extern "C" {
#endif


//*****************************************************************************
// the includes
//*****************************************************************************
// Include files for device support, F2806x in this case
#include "inc/hw_types.h"
#include "driverlib.h"
#include "device.h"
#include "pfc1ph3ilttpl_settings.h"

//*****************************************************************************
//defines
//*****************************************************************************

// Timer definitions based on System Clock
// 150 MHz devices
#define     mSec0_5     0.5*SYSTEM_FREQUENCY*1000       // 0.5 mS
#define     mSec1       1*SYSTEM_FREQUENCY*1000         // 1.0 mS
#define     mSec2       2.0*SYSTEM_FREQUENCY*1000       // 2.0 mS
#define     mSec5       5*SYSTEM_FREQUENCY*1000         // 5.0 mS
#define     mSec7_5     7.5*SYSTEM_FREQUENCY*1000       // 7.5 mS
#define     mSec10      10*SYSTEM_FREQUENCY*1000        // 10 mS
#define     mSec20      20*SYSTEM_FREQUENCY*1000        // 20 mS
#define     mSec50      50*SYSTEM_FREQUENCY*1000        // 50 mS
#define     mSec100     100*SYSTEM_FREQUENCY*1000       // 100 mS
#define     mSec500     500*SYSTEM_FREQUENCY*1000       // 500 mS
#define     mSec1000    1000*SYSTEM_FREQUENCY*1000      // 1000 mS

#ifndef TRUE
#define FALSE 0
#define TRUE  1
#endif

#define VBUS_FB     ADC_readResult(VBUS_ADCRESULTREGBASE,VBUS_ADC_SOC_NO)
#define VBUS_FB_2   ADC_readResult(VBUS_ADCRESULTREGBASE,VBUS_ADC_SOC_NO2)
#define VBUS_FB_3   ADC_readResult(VBUS_ADCRESULTREGBASE,VBUS_ADC_SOC_NO3)

#define AC_CUR_FB   ADC_readResult(AC_CUR_ADCRESULTREGBASE,AC_CUR_ADC_SOC_NO)
#define AC_CUR_FB_2 ADC_readResult(AC_CUR_ADCRESULTREGBASE,AC_CUR_ADC_SOC_NO_2)
#define AC_CUR_FB_3 ADC_readResult(AC_CUR_ADCRESULTREGBASE,AC_CUR_ADC_SOC_NO_3)

#define AC_L_FB     ADC_readResult(AC_L_ADCRESULTREGBASE,AC_L_ADC_SOC_NO)
#define AC_L_FB_2   ADC_readResult(AC_L_ADCRESULTREGBASE,AC_L_ADC_SOC_NO2)
#define AC_L_FB_3   ADC_readResult(AC_L_ADCRESULTREGBASE,AC_L_ADC_SOC_NO3)

#define AC_N_FB     ADC_readResult(AC_N_ADCRESULTREGBASE,AC_N_ADC_SOC_NO)
#define AC_N_FB_2   ADC_readResult(AC_N_ADCRESULTREGBASE,AC_N_ADC_SOC_NO2)
#define AC_N_FB_3   ADC_readResult(AC_N_ADCRESULTREGBASE,AC_N_ADC_SOC_NO3)

#define IL1_FB      ADC_readResult(IL1_ADCRESULTREGBASE,IL1_ADC_SOC_NO)
#define IL2_FB      ADC_readResult(IL2_ADCRESULTREGBASE,IL2_ADC_SOC_NO)
#define IL3_FB      ADC_readResult(IL3_ADCRESULTREGBASE,IL3_ADC_SOC_NO)

#define PWM_TRIP_STATUS EPWM_getTripZoneFlagStatus

#define HistorySize 8
//#define DLOG_SIZE   200

#define ADC_PU_SCALE_FACTOR  (float32_t)(0.000244140625)
#define ADC_PU_PPB_SCALE_FACTOR 0.000488281250 //1/2^11

//definitions for selecting DACH reference
#define REFERENCE_VDDA     0

// ADC Configuration
#define REFERENCE_INTERNAL 0 //internal reference (12-bit only)
#define REFERENCE_EXTERNAL 1 //external reference

#define REFERENCE_VDAC     0
#define REFERENCE_VREF     1

#define GET_TASK_A_TIMER_OVERFLOW_STATUS CPUTimer_getTimerOverflowStatus(CPUTIMER0_BASE)
#define CLEAR_TASK_A_TIMER_OVERFLOW_FLAG CPUTimer_clearOverflowFlag(CPUTIMER0_BASE)

#define GET_TASK_B_TIMER_OVERFLOW_STATUS CPUTimer_getTimerOverflowStatus(CPUTIMER1_BASE)
#define CLEAR_TASK_B_TIMER_OVERFLOW_FLAG CPUTimer_clearOverflowFlag(CPUTIMER1_BASE)

#define DISABLE_AQ_SW_FRC(m)                HWREGH(m + EPWM_O_AQCSFRC) = 0x00;
#define AQ_SW_FORCE_PWMxA_HIGH_PWMxB_LOW(m) HWREGH(m + EPWM_O_AQCSFRC) = 0x06;
#define AQ_SW_FORCE_PWMxA_LOW_PWMxB_HIGH(m) HWREGH(m + EPWM_O_AQCSFRC) = 0x09;
#define AQ_SW_FORCE_PWMxA_LOW_PWMxB_LOW(m)  HWREGH(m + EPWM_O_AQCSFRC) = 0x05;
#define SET_PWM_DBRED(m,n)                  HWREGH(m + EPWM_O_DBRED ) = n;
#define SET_PWM_DBFED(m,n)                  HWREGH(m + EPWM_O_DBFED ) = n;

#define ENABLE_SWAP_DEADBAND_OUTPUT(m)      HWREGH(m + EPWM_O_DBCTL) = (HWREGH(m + EPWM_O_DBCTL) | 0x3000);
#define DISABLE_SWAP_DEADBAND_OUTPUT(m)     HWREGH(m + EPWM_O_DBCTL) = (HWREGH(m + EPWM_O_DBCTL) & ~0x3000);

//*****************************************************************************
//globals
//*****************************************************************************

//*****************************************************************************
// the function prototypes
//*****************************************************************************

void setupDevice(void);
void setupPFC1ph3ilttplPWM(uint32_t base1,uint32_t base2, uint32_t base3,
                           uint32_t base4, uint16_t pwm_period_ticks,
                           uint16_t pwm_dbred_ticks, uint16_t pwm_dbfed_ticks);
void setupADCconversion(uint32_t adc_base, uint16_t adc_soc_number,
                        uint16_t adc_trig_source, uint16_t adc_pin,
                        uint16_t acqps);
void setupInverterPWMnewmethod(uint16_t inv_pwm_no, uint16_t pwm_period_ticks,
                               uint16_t pwm_dbred_ticks,
                               uint16_t pwm_dbfed_ticks);
void setupADC(void);
void setupADCaConversion(uint16_t trig_sel, uint16_t acqps, uint16_t I_pin,
                         uint16_t Vin_pin, uint16_t Vout_pin,
                         uint16_t Vref_pin);
void configurePWM1chUpDwnCnt(int16_t n, int16_t period, int16_t mode,
                             int16_t phase, int16_t db_red, int16_t db_fed,
                             int16_t phase_dir);
void configurePWM1chUpCnt(uint32_t n, uint16_t period);
void setupBoardProtection(uint32_t base1,uint32_t base2, uint32_t base3,
                          uint32_t base4, float32_t current_limit,
                          float32_t current_max_sense);
void setupCMPSS(uint32_t base1, float32_t current_limit, float32_t current_max_sense );
void initGlobalVariables(void);
void calibrateOffset(volatile float32_t *ac_cur_sensedOffset,
                     volatile float32_t *IL1c_sensedOffset,
                     volatile float32_t *IL2c_sensedOffset ,
                     volatile float32_t *IL3c_sensedOffset ,
                     float32_t k1, float32_t k2);
void setupProfilingGPIO(void);
void setupRelayGPIO(void);
void toggleLED(void);
void setupLEDGPIO(void);
void disablePWMCLKCounting(void);
void enablePWMCLKCounting(void);

void updateBoardStatus(void);
void setupGainChangeGPIO(void);
void DeviceInit(void);
void MemCopy(void);
void InitFlash(void);

void setupEPWMtoTriggerADCSOC(uint32_t base);

void setPinsAsPWM();

void setupCLA(void);

//
// Function Prototypes
//
//CLA C Tasks defined in Cla1Tasks_C.cla
__attribute__((interrupt))  void Cla1Task1();
__attribute__((interrupt))  void Cla1Task2();
__attribute__((interrupt))  void Cla1Task3();
__attribute__((interrupt))  void Cla1Task4();
__attribute__((interrupt))  void Cla1Task5();
__attribute__((interrupt))  void Cla1Task6();
__attribute__((interrupt))  void Cla1Task7();
__attribute__((interrupt))  void Cla1BackgroundTask();

extern uint16_t Cla1ProgLoadStart;
extern uint16_t Cla1ProgLoadEnd;
extern uint16_t Cla1ProgLoadSize;
extern uint16_t Cla1ProgRunStart;
extern uint16_t Cla1ProgRunEnd;
extern uint16_t Cla1ProgRunSize;

// ISR related
#if CONTROL_RUNNING_ON == C28x_CORE

#ifndef __TMS320C28XX_CLA__
    #pragma INTERRUPT (controlISR, HPI)
    #pragma CODE_SECTION(controlISR,"isrcodefuncs");
    interrupt void controlISR(void);
    inline void clearInterrupt(uint16_t pie_group_no);
    inline void setupInterrupt(void);
#endif

#endif

#if INSTRUMENTATION_ISR_RUNNING_ON == C28x_CORE

#ifndef __TMS320C28XX_CLA__
    #pragma CODE_SECTION(tenKHzISR,"ramfuncs");
    interrupt void tenKHzISR(void);
    inline void clearInterrupt(uint16_t pie_group_no);
    inline void setupInterrupt(void);
#endif

#endif

//*****************************************************************************
// Inline functions
//*****************************************************************************

static inline void EPWM_setCounterCompareValueOptimized(uint32_t base,
                                 EPWM_CounterCompareModule compModule,
                                 uint16_t compCount)
{
    uint32_t registerOffset;

    // Get the register offset for the Counter compare
    registerOffset = EPWM_O_CMPA + (uint16_t)compModule;
    // Write to the counter compare registers.
    #pragma diag_suppress=770
    #pragma diag_suppress=173
    if((compModule == EPWM_COUNTER_COMPARE_A) ||
    (compModule == EPWM_COUNTER_COMPARE_B))
    {
        // write to CMPA or COMPB bits
        HWREGH(base + registerOffset + 1) = (uint16_t)compCount;
    }
    else
    { // write to COMPC or COMPD bits
        HWREGH(base + registerOffset) = compCount;
    }
    #pragma diag_warning=770
    #pragma diag_warning=173
}


//TODO updateInverterPWM()
inline void update_pfc1ph3ilttplPWM(uint32_t base1, uint32_t base2,
                                    uint32_t base3, uint32_t base4,
                                    float32_t duty, float32_t phase1ScaleFactor,
                                    float32_t phase2ScaleFactor,
                                    float32_t phase3ScaleFactor)
{
    uint32_t dutyPWMReg;

    dutyPWMReg= (uint32_t)((float32_t)(PFC_PWM_PERIOD/2.0) * fabsf(duty));

    EPWM_setCounterCompareValueOptimized(base2,EPWM_COUNTER_COMPARE_A,
                   (uint32_t)((float32_t)dutyPWMReg*(float32_t)phase1ScaleFactor));
    EPWM_setCounterCompareValueOptimized(base3,EPWM_COUNTER_COMPARE_A,
                   (uint32_t)((float32_t)dutyPWMReg*(float32_t)phase2ScaleFactor));
    EPWM_setCounterCompareValueOptimized(base4,EPWM_COUNTER_COMPARE_A,
                   (uint32_t)((float32_t)dutyPWMReg*(float32_t)phase3ScaleFactor));

}

//TODO closeRelay
inline void closeRelay(void)
{
    GPIO_writePin(GPIO_RELAY,1);
}

//TODO openRelay
inline void openRelay(void)
{
    GPIO_writePin(GPIO_RELAY,0);
}

//TODO gainHighSetforLowCurrent
inline void gainHighSetforLowCurrent(void)
{
    GPIO_writePin(GPIO_I_SENSE_GAIN_ADJUST,1);
}

//TODO gainLowSetforHighCurrent
inline void gainLowSetforHighCurrent(void)
{
    GPIO_writePin(GPIO_I_SENSE_GAIN_ADJUST,0);
}

//TODO setProfilingGPIO
inline void setProfilingGPIO(void)
{
    #pragma diag_suppress=770
    #pragma diag_suppress=173
    HWREG(GPIODATA_BASE  + GPIO_O_GPASET ) = GPIO_PROFILING1_SET;
    #pragma diag_warning=770
    #pragma diag_warning=173
}

//TODO resetProfilingGPIO
inline void resetProfilingGPIO(void)
{
    #pragma diag_suppress=770
    #pragma diag_suppress=173
    HWREG(GPIODATA_BASE  + GPIO_O_GPACLEAR ) = GPIO_PROFILING1_CLEAR;
    #pragma diag_warning=770
    #pragma diag_warning=173
}

//TODO setProfilingGPIO
inline void setProfilingGPIO2(void)
{
    #pragma diag_suppress=770
    #pragma diag_suppress=173
    HWREG(GPIODATA_BASE  + GPIO_O_GPASET ) = GPIO_PROFILING2_SET;
    #pragma diag_warning=770
    #pragma diag_warning=173
}

//TODO resetProfilingGPIO
inline void resetProfilingGPIO2(void)
{
    #pragma diag_suppress=770
    #pragma diag_suppress=173
    HWREG(GPIODATA_BASE  + GPIO_O_GPACLEAR ) = GPIO_PROFILING2_CLEAR;
    #pragma diag_warning=770
    #pragma diag_warning=173
}

//TODO clearPWMTrip
inline void clearPWMTripFlags(uint32_t base)
{
    // clear all the configured trip sources for the PWM module
    EPWM_clearTripZoneFlag(base,
                           ( EPWM_TZ_INTERRUPT_OST |
                             EPWM_TZ_INTERRUPT_CBC |
                             EPWM_TZ_INTERRUPT_DCAEVT1 )
                           );
}

inline void clearOSTPWMTripFlag(uint32_t base)
{
    // clear all the configured trip sources for the PWM module
    EPWM_clearTripZoneFlag(base,EPWM_TZ_INTERRUPT_OST);
}

inline void forceOSTPWMTrip(uint32_t base)
{
    EPWM_forceTripZoneEvent(base,EPWM_TZ_FORCE_EVENT_OST);
}

//TODO clearPWM Interrupt Flag
inline void clearPWMInterruptFlag(uint32_t base)
{
    EPWM_clearEventTriggerInterruptFlag(base);
}

//TODO enable PWM Interrupt generation
inline void enablePWMInterruptGeneration(uint32_t base)
{
    EPWM_setInterruptSource(base,EPWM_INT_TBCTR_D_CMPB);
    EPWM_setInterruptEventCount(base,CNTRL_ISR_FREQ_RATIO);
    EPWM_enableInterrupt(base);
    EPWM_clearEventTriggerInterruptFlag(base);
}


#ifndef __TMS320C28XX_CLA__
//TODO clearInterrupt
inline void clearInterrupt(uint16_t pie_group_no)
{
    Interrupt_clearACKGroup(pie_group_no);
}


inline void setupInterrupt(void)
{
    CPUTimer_enableInterrupt(
            C28x_INSTRUMENTATION_INTERRUPT_TRIG_CPUTIMER_BASE
            );
    CPUTimer_clearOverflowFlag(
            C28x_INSTRUMENTATION_INTERRUPT_TRIG_CPUTIMER_BASE
            );

#if CONTROL_RUNNING_ON == C28x_CORE
    Interrupt_register(C28x_CONTROLISR_INTERRUPT, &controlISR);
    enablePWMInterruptGeneration(C28x_CONTROLISR_INTERRUPT_TRIG_PWM_BASE);
    clearInterrupt(C28x_CONTROLISR_INTERRUPT_PIE_GROUP_NO);
    Interrupt_enable(C28x_CONTROLISR_INTERRUPT);
#endif

#if INSTRUMENTATION_ISR_RUNNING_ON == C28x_CORE
    Interrupt_register(C28x_INSTRUMENTATION_INTERRUPT, &tenKHzISR);
    Interrupt_enable(C28x_INSTRUMENTATION_INTERRUPT);
#endif

#if (CONTROL_RUNNING_ON==CLA_CORE || INSTRUMENTATION_ISR_RUNNING_ON==CLA_CORE)
    setupCLA();
#endif

    EALLOW;
    EINT;  // Enable Global interrupt INTM
    ERTM;  // Enable Global realtime interrupt DBGM
    EDIS;
}

#endif

#ifdef __cplusplus
}
#endif                                  /* extern "C" */


#endif
